from binance.lib.utils import check_required_parameter
from binance.lib.utils import check_required_parameters


def new_order_test(self, symbol: str, side: str, type: str, **kwargs):
    """Test New Order (TRADE)

    Test new order creation and signature/recvWindow. Creates and validates a new order but does not send it into
    the matching engine.

    POST /api/v3/order/test

    https://binance-docs.github.io/apidocs/spot/en/#test-new-order-trade

    Args:
        symbol (str)
        side (str)
        type (str)
    Keyword Args:
        timeInForce (str, optional)
        quantity (float, optional)
        quoteOrderQty (float, optional)
        price (float, optional)
        newClientOrderId (str, optional): A unique id among open orders. Automatically generated if not sent.
        stopPrice (float, optional): Used with STOP_LOSS, STOP_LOSS_LIMIT, TAKE_PROFIT, and TAKE_PROFIT_LIMIT orders.
        icebergQty (float, optional): Used with LIMIT, STOP_LOSS_LIMIT, and TAKE_PROFIT_LIMIT to create an iceberg order.
        newOrderRespType (str, optional): Set the response JSON. ACK, RESULT, or FULL;
                MARKET and LIMIT order types default to FULL, all other orders default to ACK.
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[symbol, "symbol"], [side, "side"], [type, "type"]])
    params = {"symbol": symbol, "side": side, "type": type, **kwargs}
    url_path = "/api/v3/order/test"
    return self.sign_request("POST", url_path, params)


def new_order(self, symbol: str, side: str, type: str, **kwargs):
    """New Order (TRADE)

    Post a new order

    POST /api/v3/order

    https://binance-docs.github.io/apidocs/spot/en/#new-order-trade

    Args:
        symbol (str)
        side (str)
        type (str)
    Keyword Args:
        timeInForce (str, optional)
        quantity (float, optional)
        quoteOrderQty (float, optional)
        price (float, optional)
        newClientOrderId (str, optional): A unique id among open orders. Automatically generated if not sent.
        stopPrice (float, optional): Used with STOP_LOSS, STOP_LOSS_LIMIT, TAKE_PROFIT, and TAKE_PROFIT_LIMIT orders.
        icebergQty (float, optional): Used with LIMIT, STOP_LOSS_LIMIT, and TAKE_PROFIT_LIMIT to create an iceberg order.
        newOrderRespType (str, optional): Set the response JSON. ACK, RESULT, or FULL;
                MARKET and LIMIT order types default to FULL, all other orders default to ACK.
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[symbol, "symbol"], [side, "side"], [type, "type"]])
    params = {"symbol": symbol, "side": side, "type": type, **kwargs}
    url_path = "/api/v3/order"
    return self.sign_request("POST", url_path, params)


def cancel_order(self, symbol: str, **kwargs):
    """Cancel Order (TRADE)

    Cancel an active order.

    DELETE /api/v3/order

    https://binance-docs.github.io/apidocs/spot/en/#cancel-order-trade

    Args:
        symbol (str)
    Keyword Args:
        orderId (int, optional)
        origClientOrderId (str, optional)
        newClientOrderId (str, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(symbol, "symbol")

    url_path = "/api/v3/order"
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("DELETE", url_path, payload)


def cancel_open_orders(self, symbol: str, **kwargs):
    """Cancel all Open Orders on a Symbol (TRADE)

    Cancels all active orders on a symbol.
    This includes OCO orders.

    DELETE api/v3/openOrders

    https://binance-docs.github.io/apidocs/spot/en/#cancel-all-open-orders-on-a-symbol-trade

    Args:
        symbol (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(symbol, "symbol")

    url_path = "/api/v3/openOrders"
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("DELETE", url_path, payload)


def get_order(self, symbol, **kwargs):
    """Query Order (USER_DATA)

    Check an order's status.

    GET /api/v3/order

    https://binance-docs.github.io/apidocs/spot/en/#query-order-user_data

    Args:
        symbol (str)
    Keyword Args:
        orderId (int, optional)
        origClientOrderId (str, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(symbol, "symbol")

    url_path = "/api/v3/order"
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("GET", url_path, payload)


def get_open_orders(self, symbol=None, **kwargs):
    """Current Open Orders (USER_DATA)

    Get all open orders on a symbol.

    GET /api/v3/openOrders

    https://binance-docs.github.io/apidocs/spot/en/#current-open-orders-user_data

    Args:
        symbol (str, optional)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    url_path = "/api/v3/openOrders"
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("GET", url_path, payload)


def get_orders(self, symbol: str, **kwargs):
    """All Orders (USER_DATA)

    Get all account orders; active, canceled, or filled.

    GET /api/v3/allOrders

    https://binance-docs.github.io/apidocs/spot/en/#all-orders-user_data

    Args:
        symbol (str)
    Keyword Args:
        orderId (int, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): Default 500; max 1000.
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(symbol, "symbol")

    url_path = "/api/v3/allOrders"
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("GET", url_path, payload)


def new_oco_order(
    self,
    symbol: str,
    side: str,
    quantity: float,
    price: float,
    stopPrice: float,
    **kwargs
):
    """New OCO (TRADE)

    Post a new oco order

    POST /api/v3/order/oco

    https://binance-docs.github.io/apidocs/spot/en/#new-oco-trade

    Args:
        symbol (str)
        side (str)
        quantity (float)
        price (float)
        stopPrice (float)
    Keyword Args:
        listClientOrderId (str, optional): A unique Id for the entire orderList
        limitClientOrderId (str, optional)
        limitIcebergQty (float, optional)
        stopClientOrderId (str, optional)
        stopLimitPrice (float, optional)
        stopIcebergQty (float, optional)
        stopLimitTimeInForce (str, optional)
        newOrderRespType (str, optional): Set the response JSON.
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [
            [symbol, "symbol"],
            [side, "side"],
            [quantity, "quantity"],
            [price, "price"],
            [stopPrice, "stopPrice"],
        ]
    )
    params = {
        "symbol": symbol,
        "side": side,
        "quantity": quantity,
        "price": price,
        "stopPrice": stopPrice,
        **kwargs,
    }

    url_path = "/api/v3/order/oco"
    return self.sign_request("POST", url_path, params)


def cancel_oco_order(self, symbol, **kwargs):
    """Cancel OCO (TRADE)

    Cancel an entire Order List

    DELETE /api/v3/orderList

    https://binance-docs.github.io/apidocs/spot/en/#cancel-oco-trade

    Args:
        symbol (str)
    Keyword Args:
        orderListId (int, optional): Either orderListId or listClientOrderId must be provided
        listClientOrderId (str, optional): Either orderListId or listClientOrderId must be provided
        newClientOrderId (str, optional): Used to uniquely identify this cancel. Automatically generated by default.
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(symbol, "symbol")

    url_path = "/api/v3/orderList"
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("DELETE", url_path, payload)


def get_oco_order(self, **kwargs):
    """Query OCO (USER_DATA)

    Retrieves a specific OCO based on provided optional parameters

    GET /api/v3/orderList

    https://binance-docs.github.io/apidocs/spot/en/#query-oco-user_data

    Keyword Args:
        orderListId (int, optional): Either orderListId or listClientOrderId must be provided
        origClientOrderId (str, optional): Either orderListId or listClientOrderId must be provided.
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    url_path = "/api/v3/orderList"
    return self.sign_request("GET", url_path, {**kwargs})


def get_oco_orders(self, **kwargs):
    """Query all OCO (USER_DATA)

    Retrieves all OCO based on provided optional parameters

    GET /api/v3/allOrderList

    https://binance-docs.github.io/apidocs/spot/en/#query-all-oco-user_data
    Keyword Args:
        fromId (int, optional): If supplied, neither startTime or endTime can be provided
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): Default Value: 500; Max Value: 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    url_path = "/api/v3/allOrderList"
    return self.sign_request("GET", url_path, {**kwargs})


def get_oco_open_orders(self, **kwargs):
    """Query Open OCO (USER_DATA)

    GET /api/v3/openOrderList

    https://binance-docs.github.io/apidocs/spot/en/#query-open-oco-user_data
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    url_path = "/api/v3/openOrderList"
    return self.sign_request("GET", url_path, {**kwargs})


def account(self, **kwargs):
    """Account Information (USER_DATA)

    Get current account information

    GET /api/v3/account

    https://binance-docs.github.io/apidocs/spot/en/#account-information-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    url_path = "/api/v3/account"
    return self.sign_request("GET", url_path, {**kwargs})


def my_trades(self, symbol: str, **kwargs):
    """Account Trade List (USER_DATA)

    Get trades for a specific account and symbol.

    GET /api/v3/myTrades

    https://binance-docs.github.io/apidocs/spot/en/#account-trade-list-user_data

    Args:
        symbol (str)
    Keyword Args:
        fromId (int, optional): TradeId to fetch from. Default gets most recent trades.
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): Default Value: 500; Max Value: 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(symbol, "symbol")

    url_path = "/api/v3/myTrades"
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("GET", url_path, payload)



from binance.lib.utils import check_required_parameters


def mining_algo_list(self):
    """Acquiring Algorithm (MARKET_DATA)

    GET /sapi/v1/mining/pub/algoList

    https://binance-docs.github.io/apidocs/spot/en/#acquiring-algorithm-market_data

    """

    return self.limit_request("GET", "/sapi/v1/mining/pub/algoList")


def mining_coin_list(self):
    """Acquiring CoinName (MARKET_DATA)

    GET /sapi/v1/mining/pub/coinList

    https://binance-docs.github.io/apidocs/spot/en/#acquiring-coinname-market_data

    """

    return self.limit_request("GET", "/sapi/v1/mining/pub/coinList")


def mining_worker(self, algo: str, userName: str, workerName: str, **kwargs):
    """Request for Detail Miner List (USER_DATA)

    GET /sapi/v1/mining/worker/detail

    https://binance-docs.github.io/apidocs/spot/en/#request-for-detail-miner-list-user_data

    Args:
        algo (str)
        userName (str): Mining account
        workerName (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters(
        [[algo, "algo"], [userName, "userName"], [workerName, "workerName"]]
    )

    payload = {"algo": algo, "userName": userName, "workerName": workerName, **kwargs}
    return self.sign_request("GET", "/sapi/v1/mining/worker/detail", payload)


def mining_worker_list(self, algo: str, userName: str, **kwargs):
    """Request for Miner List (USER_DATA)

    GET /sapi/v1/mining/worker/list

    https://binance-docs.github.io/apidocs/spot/en/#request-for-miner-list-user_data

    Args:
        algo (str)
        userName (str): Mining account
    Keyword Args:
        pageIndex (int, optional): Page number，default is first page，start form 1
        sort (int, optional): sort sequence(default=0)0 positive sequence，1 negative sequence
        sortColumn (int, optional): Sort by( default 1):
                1: miner name, 2: real-time computing power, 3: daily average computing power,
                4: real-time rejection rate, 5: last submission time
        workerStatus (int, optional): miners status (default = 0) 0 all, 1 valid, 2 invalid, 3 failure
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[algo, "algo"], [userName, "userName"]])

    payload = {"algo": algo, "userName": userName, **kwargs}
    return self.sign_request("GET", "/sapi/v1/mining/worker/list", payload)


def mining_earnings_list(self, algo: str, userName: str, **kwargs):
    """Revenue List (USER_DATA)

    GET /sapi/v1/mining/payment/list

    https://binance-docs.github.io/apidocs/spot/en/#earnings-list-user_data

    Args:
        algo (str)
        userName (str): Mining account
    Keyword Args:
        coin (str, optional): Coin Name
        startDate (int, optional): Search date, millisecond timestamp, while empty query all
        endDate (int, optional): Search date, millisecond timestamp, while empty query all
        pageIndex (int, optional): Page number, empty default first page, starting from 1
        pageSize (int, optional): Number of pages, minimum 10, maximum 200
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[algo, "algo"], [userName, "userName"]])

    payload = {"algo": algo, "userName": userName, **kwargs}
    return self.sign_request("GET", "/sapi/v1/mining/payment/list", payload)


def mining_bonus_list(self, algo: str, userName: str, **kwargs):
    """Extra Bonus List (USER_DATA)

    GET /sapi/v1/mining/payment/other

    https://binance-docs.github.io/apidocs/spot/en/#extra-bonus-list-user_data

    Args:
        algo (str)
        userName (str): Mining account
    Keyword Args:
        coin (str, optional): Coin Name
        startDate (int, optional): Search date, millisecond timestamp, while empty query all
        endDate (int, optional): Search date, millisecond timestamp, while empty query all
        pageIndex (int, optional): Page number, empty default first page, starting from 1
        pageSize (int, optional): Number of pages, minimum 10, maximum 200
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[algo, "algo"], [userName, "userName"]])

    payload = {"algo": algo, "userName": userName, **kwargs}
    return self.sign_request("GET", "/sapi/v1/mining/payment/other", payload)


def mining_statistics_list(self, algo: str, userName: str, **kwargs):
    """Statistic List (USER_DATA)

    GET /sapi/v1/mining/statistics/user/status

    https://binance-docs.github.io/apidocs/spot/en/#statistic-list-user_data

    Args:
        algo (str)
        userName (str): Mining account
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000

    """
    check_required_parameters([[algo, "algo"], [userName, "userName"]])

    payload = {"algo": algo, "userName": userName, **kwargs}
    return self.sign_request("GET", "/sapi/v1/mining/statistics/user/status", payload)


def mining_account_list(self, algo: str, userName: str, **kwargs):
    """Account List (USER_DATA)

    GET /sapi/v1/mining/statistics/user/list

    https://binance-docs.github.io/apidocs/spot/en/#account-list-user_data

    Args:
        algo (str)
        userName (str): Mining account
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000

    """
    check_required_parameters([[algo, "algo"], [userName, "userName"]])

    payload = {"algo": algo, "userName": userName, **kwargs}
    return self.sign_request("GET", "/sapi/v1/mining/statistics/user/list", payload)


def mining_hashrate_resale_request(
    self,
    algo: str,
    userName: str,
    startDate: int,
    endDate: int,
    toPoolUser: str,
    hashRate: int,
    **kwargs
):
    """Hashrate Resale Request (USER_DATA)

    POST /sapi/v1/mining/hash-transfer/config

    https://binance-docs.github.io/apidocs/spot/en/#hashrate-resale-request-user_data

    Args:
        algo (str)
        userName (str): Mining account
        startDate (int): Resale Start Time(Millisecond timestamp)
        endDate (int): Resale End Time (Millisecond timestamp)
        toPoolUser (str): Mining Account
        hashRate (int): Resale hashrate h/s must be transferred
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [
            [algo, "algo"],
            [userName, "userName"],
            [startDate, "startDate"],
            [endDate, "endDate"],
            [toPoolUser, "toPoolUser"],
            [hashRate, "hashRate"],
        ]
    )

    payload = {
        "algo": algo,
        "userName": userName,
        "startDate": startDate,
        "endDate": endDate,
        "toPoolUser": toPoolUser,
        "hashRate": hashRate,
        **kwargs,
    }
    return self.sign_request("POST", "/sapi/v1/mining/hash-transfer/config", payload)


def mining_hashrate_resale_cancellation(self, configId: int, userName: str, **kwargs):
    """Cancel hashrate resale configuration(USER_DATA)

    POST /sapi/v1/mining/hash-transfer/config/cancel

    https://binance-docs.github.io/apidocs/spot/en/#cancel-hashrate-resale-configuration-user_data

    Args:
        configId (int): Mining ID
        userName (str): Mining account
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[configId, "configId"], [userName, "userName"]])

    payload = {"configId": configId, "userName": userName, **kwargs}
    return self.sign_request(
        "POST", "/sapi/v1/mining/hash-transfer/config/cancel", payload
    )


def mining_hashrate_resale_list(self, **kwargs):
    """Hashrate Resale List (USER_DATA)

    GET /sapi/v1/mining/hash-transfer/config/details/list

    https://binance-docs.github.io/apidocs/spot/en/#hashrate-resale-list-user_data

    Keyword Args:
        pageIndex (int, optional): Page number, empty default first page, starting from 1
        pageSize (int, optional): Number of pages, minimum 10, maximum 200
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request(
        "GET", "/sapi/v1/mining/hash-transfer/config/details/list", kwargs
    )


def mining_hashrate_resale_details(self, configId: int, userName: str, **kwargs):
    """Hashrate Resale Detail (USER_DATA)

    GET /sapi/v1/mining/hash-transfer/profit/details

    https://binance-docs.github.io/apidocs/spot/en/#hashrate-resale-detail-user_data

    Args:
        configId (int): Mining ID
        userName (str): Mining account
    Keyword Args:
        pageIndex (int, optional): Page number, empty default first page, starting from 1
        pageSize (int, optional): Number of pages, minimum 10, maximum 200
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[configId, "configId"], [userName, "userName"]])

    payload = {"configId": configId, "userName": userName, **kwargs}
    return self.sign_request(
        "GET", "/sapi/v1/mining/hash-transfer/profit/details", payload
    )



import json
import logging
from autobahn.twisted.websocket import WebSocketClientProtocol


class BinanceClientProtocol(WebSocketClientProtocol):
    def __init__(self, factory, payload=None):
        super().__init__()
        self.factory = factory
        self.payload = payload

    def onOpen(self):
        self.factory.protocol_instance = self

    def onConnect(self, response):
        logging.info("Server connected")
        if self.payload:
            logging.info("Sending message to Server: {}".format(self.payload))
            self.sendMessage(self.payload, isBinary=False)
        # reset the delay after reconnecting
        self.factory.resetDelay()

    def onMessage(self, payload, isBinary):
        if not isBinary:
            try:
                payload_obj = json.loads(payload.decode("utf8"))
            except ValueError:
                pass
            else:
                self.factory.callback(payload_obj)

    def onClose(self, wasClean, code, reason):
        logging.warn(
            "WebSocket connection closed: {0}, code: {1}, clean: {2}, reason: {0}".format(
                reason, code, wasClean
            )
        )

    def onPing(self, payload):
        logging.info("Received Ping from server")
        self.sendPong()
        logging.info("Responded Pong to server")

    def onPong(self, payload):
        logging.info("Received Pong from server")



from binance.lib.utils import check_required_parameter, check_required_parameters


def new_listen_key(self):
    """Create a ListenKey (USER_STREAM)

    POST /api/v3/userDataStream

    https://binance-docs.github.io/apidocs/spot/en/#listen-key-spot
    """

    url_path = "/api/v3/userDataStream"
    return self.send_request("POST", url_path)


def renew_listen_key(self, listenKey: str):
    """Create a ListenKey (USER_STREAM)

    PUT /api/v3/userDataStream

    https://binance-docs.github.io/apidocs/spot/en/#listen-key-spot

    Args:
        listenKey (str)
    """
    check_required_parameter(listenKey, "listenKey")

    url_path = "/api/v3/userDataStream"
    return self.send_request("PUT", url_path, {"listenKey": listenKey})


def close_listen_key(self, listenKey: str):
    """Close a ListenKey (USER_STREAM)

    DELETE /api/v3/userDataStream

    https://binance-docs.github.io/apidocs/spot/en/#listen-key-spot

    Args:
        listenKey (str)
    """
    check_required_parameter(listenKey, "listenKey")

    url_path = "/api/v3/userDataStream"
    return self.send_request("DELETE", url_path, {"listenKey": listenKey})


# Margin
def new_margin_listen_key(self):
    """Create a margin ListenKey (USER_STREAM)

    POST /sapi/v1/userDataStream

    https://binance-docs.github.io/apidocs/spot/en/#listen-key-margin
    """

    url_path = "/sapi/v1/userDataStream"
    return self.send_request("POST", url_path)


def renew_margin_listen_key(self, listenKey: str):
    """Renew a margin ListenKey (USER_STREAM)

    PUT /sapi/v1/userDataStream

    https://binance-docs.github.io/apidocs/spot/en/#listen-key-margin

    Args:
        listenKey (str)
    """
    check_required_parameter(listenKey, "listenKey")

    url_path = "/sapi/v1/userDataStream"
    return self.send_request("PUT", url_path, {"listenKey": listenKey})


def close_margin_listen_key(self, listenKey: str):
    """Close a margin ListenKey (USER_STREAM)

    DELETE /sapi/v1/userDataStream

    https://binance-docs.github.io/apidocs/spot/en/#listen-key-margin

    Args:
        listenKey (str)
    """
    check_required_parameter(listenKey, "listenKey")

    url_path = "/sapi/v1/userDataStream"
    return self.send_request("DELETE", url_path, {"listenKey": listenKey})


# isolated margin
def new_isolated_margin_listen_key(self, symbol: str):
    """Create an isolated margin ListenKey (USER_STREAM)

    POST /sapi/v1/userDataStream/isolated

    https://binance-docs.github.io/apidocs/spot/en/#listen-key-margin

    Args:
        symbol (str)
    """
    check_required_parameter(symbol, "symbol")

    url_path = "/sapi/v1/userDataStream/isolated"
    return self.send_request("POST", url_path, {"symbol": symbol})


def renew_isolated_margin_listen_key(self, listenKey: str, symbol: str):
    """Renew an isolated ListenKey (USER_STREAM)

    PUT /sapi/v1/userDataStream/isolated

    https://binance-docs.github.io/apidocs/spot/en/#listen-key-margin

    Args:
        listenKey (str)
        symbol (str)
    """

    check_required_parameters([[listenKey, "listenKey"], [symbol, "symbol"]])

    payload = {"listenKey": listenKey, "symbol": symbol}

    url_path = "/sapi/v1/userDataStream/isolated"
    return self.send_request("PUT", url_path, payload)


def close_isolated_margin_listen_key(self, listenKey: str, symbol: str):
    """Close an isolated margin ListenKey (USER_STREAM)

    DELETE /sapi/v1/userDataStream/isolated

    https://binance-docs.github.io/apidocs/spot/en/#listen-key-margin

    Args:
        listenKey (str)
        symbol (str)
    """

    check_required_parameters([[listenKey, "listenKey"], [symbol, "symbol"]])

    payload = {"listenKey": listenKey, "symbol": symbol}

    url_path = "/sapi/v1/userDataStream/isolated"
    return self.send_request("DELETE", url_path, payload)



from binance.lib.utils import check_required_parameters


def blvt_info(self, tokenName: str = None):
    """Get BLVT Info (MARKET_DATA)

    GET /sapi/v1/blvt/tokenInfo

    https://binance-docs.github.io/apidocs/spot/en/#get-blvt-info-market_data

    Args:
        tokenName (str, optional): BTCDOWN, BTCUP
    """
    payload = {"tokenName": tokenName}

    return self.limit_request("GET", "/sapi/v1/blvt/tokenInfo", payload)


def subscribe_blvt(self, tokenName: str, cost, **kwargs):
    """Subscribe BLVT (USER_DATA)

    POST /sapi/v1/blvt/subscribe (HMAC SHA256)

    https://binance-docs.github.io/apidocs/spot/en/#subscribe-blvt-user_data

    Args:
        tokenName (str): BTCDOWN, BTCUP.
        cost (str): spot balance.
    Keyword Args:
        recvWindow (int, optional)
    """

    check_required_parameters([[tokenName, "tokenName"], [cost, "cost"]])

    payload = {"tokenName": tokenName, "cost": cost, **kwargs}
    return self.sign_request("POST", "/sapi/v1/blvt/subscribe", payload)


def subscription_record(self, **kwargs):
    """Query Subscription Record (USER_DATA)

    GET /sapi/v1/blvt/subscribe/record (HMAC SHA256)

    https://binance-docs.github.io/apidocs/spot/en/#query-subscription-record-user_data

    Keyword Args:
        tokenName (str, optional): BTCDOWN, BTCUP.
        id (int, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 1000, max 1000.
        recvWindow (int, optional)
    """

    return self.sign_request("GET", "/sapi/v1/blvt/subscribe/record", kwargs)


def redeem_blvt(self, tokenName: str, amount, **kwargs):
    """Redeem BLVT (USER_DATA)

    POST /sapi/v1/blvt/redeem (HMAC SHA256)

    https://binance-docs.github.io/apidocs/spot/en/#redeem-blvt-user_data

    Args:
        tokenName (str): BTCDOWN, BTCUP
        amount (str)
    Keyword Args:
        recvWindow (int, optional)
    """

    check_required_parameters([[tokenName, "tokenName"], [amount, "amount"]])

    payload = {"tokenName": tokenName, "amount": amount, **kwargs}
    return self.sign_request("POST", "/sapi/v1/blvt/redeem", payload)


def redemption_record(self, **kwargs):
    """Query Redemption Record (USER_DATA)

    GET /sapi/v1/blvt/redeem/record (HMAC SHA256)

    https://binance-docs.github.io/apidocs/spot/en/#query-redemption-record-user_data

    Keyword Args:
        tokenName (str, optional): BTCDOWN, BTCUP
        id (int, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 1000, max 1000
        recvWindow (int, optional)
    """

    return self.sign_request("GET", "/sapi/v1/blvt/redeem/record", kwargs)


def user_limit_info(self, **kwargs):
    """Get BLVT User Limit Info

    GET /sapi/v1/blvt/userLimit

    https://binance-docs.github.io/apidocs/spot/en/#get-blvt-user-limit-info

    Keyword Args:
        tokenName (str, optional): BTCDOWN, BTCUP
        recvWindow (int, optional)
    """

    return self.sign_request("GET", "/sapi/v1/blvt/userLimit", kwargs)



import time

from urllib.parse import urlencode
from binance.error import ParameterRequiredError, ParameterValueError


def cleanNoneValue(d) -> dict:
    out = {}
    for k in d.keys():
        if d[k] is not None:
            out[k] = d[k]
    return out


def check_required_parameter(value, name):
    if not value:
        raise ParameterRequiredError([name])


def check_required_parameters(params):
    """validate multiple parameters
    params = [
        ['btcusdt', 'symbol'],
        [10, 'price']
    ]

    """
    for p in params:
        check_required_parameter(p[0], p[1])


def check_enum_parameter(value, enum_class):
    if value not in set(item.value for item in enum_class):
        raise ParameterValueError([value])


def get_timestamp():
    return int(time.time() * 1000)


def encoded_string(query):
    return urlencode(query, True).replace("%40", "@")


def config_logging(logging, logging_devel, log_file=None):
    logging.basicConfig(level=logging_devel, filename=log_file)



from binance.lib.utils import check_required_parameters

"""
  Endpoints that for moving funds between spot and futures accounts.
  It's not for trading on futures.

"""


def futures_transfer(self, asset: str, amount: float, type: int, **kwargs):
    """New Future Account Transfer (FUTURES)
    Execute transfer between spot account and futures account.

    POST /sapi/v1/futures/transfer

    https://binance-docs.github.io/apidocs/spot/en/#new-future-account-transfer-futures

    Args:
        asset (str): The asset being transferred, e.g. USDT
        amount (float): The amount to be transferred
        type (int): 1: transfer from spot account to USDT-Ⓜ futures account.
              2: transfer from USDT-Ⓜ futures account to spot account.
              3: transfer from spot account to COIN-Ⓜ futures account.
              4: transfer from COIN-Ⓜ futures account to spot account.
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[asset, "asset"], [amount, "amount"], [type, "type"]])

    payload = {"asset": asset, "amount": amount, "type": type, **kwargs}
    return self.sign_request("POST", "/sapi/v1/futures/transfer", payload)


def futures_transfer_history(self, asset: str, startTime, **kwargs):
    """Get Future Account Transaction History List (USER_DATA)

    GET /sapi/v1/futures/transfer

    https://binance-docs.github.io/apidocs/spot/en/#get-future-account-transaction-history-list-user_data

    Args:
        asset (str): The asset being transferred, e.g. USDT
        startTime (int)
    Keyword Args:
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[asset, "asset"], [startTime, "startTime"]])

    payload = {"asset": asset, "startTime": startTime, **kwargs}
    return self.sign_request("GET", "/sapi/v1/futures/transfer", payload)


def futures_loan_borrow(self, coin: str, collateralCoin: str, **kwargs):
    """Borrow For Cross-Collateral (TRADE)

    POST /sapi/v1/futures/loan/borrow

    https://binance-docs.github.io/apidocs/spot/en/#borrow-for-cross-collateral-trade

    Args:
        coin (str)
        collateralCoin (str)
    Keyword Args:
        amount (float, optional): mandatory when collateralAmount is empty
        collateralAmount (float, optional): mandatory when amount is empty
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[coin, "coin"], [collateralCoin, "collateralCoin"]])

    payload = {"coin": coin, "collateralCoin": collateralCoin, **kwargs}
    return self.sign_request("POST", "/sapi/v1/futures/loan/borrow", payload)


def futures_loan_borrow_history(self, **kwargs):
    """Cross-Collateral Borrow History (USER_DATA)

    GET /sapi/v1/futures/loan/borrow/history

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-borrow-history-user_data

    Keyword Args:
        coin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/futures/loan/borrow/history", kwargs)


def futures_loan_repay(self, coin: str, collateralCoin: str, amount: float, **kwargs):
    """Repay For Cross-Collateral (TRADE)

    POST /sapi/v1/futures/loan/repay

    https://binance-docs.github.io/apidocs/spot/en/#repay-for-cross-collateral-trade

    Args:
        coin (str)
        collateralCoin (str)
        amount (float)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [[coin, "coin"], [collateralCoin, "collateralCoin"], [amount, "amount"]]
    )

    payload = {
        "coin": coin,
        "collateralCoin": collateralCoin,
        "amount": amount,
        **kwargs,
    }
    return self.sign_request("POST", "/sapi/v1/futures/loan/repay", payload)


def futures_loan_repay_history(self, **kwargs):
    """Cross-Collateral Repayment History (USER_DATA)

    GET /sapi/v1/futures/loan/repay/history

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-repayment-history-user_data

    Keyword Args:
        coin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/futures/loan/repay/history", kwargs)


def futures_loan_wallet(self, **kwargs):
    """Cross-Collateral Wallet (USER_DATA)

    GET /sapi/v2/futures/loan/wallet

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-wallet-v2-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v2/futures/loan/wallet", kwargs)


def futures_loan_configs(self, **kwargs):
    """Cross-Collateral Information (USER_DATA)

    GET /sapi/v2/futures/loan/configs

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-information-v2-user_data

    Keyword Args:
        loanCoin (str, optional)
        collateralCoin (str, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v2/futures/loan/configs", kwargs)


def futures_loan_calc_adjust_level(
    self, loanCoin: str, collateralCoin: str, amount: float, direction: str, **kwargs
):
    """Calculate Rate After Adjust Cross-Collateral LTV (USER_DATA)

    GET /sapi/v2/futures/loan/calcAdjustLevel

    https://binance-docs.github.io/apidocs/spot/en/#calculate-rate-after-adjust-cross-collateral-ltv-user_data

    Args:
        loanCoin (str)
        collateralCoin (str)
        amount (float)
        direction (str): "ADDITIONAL", "REDUCED"
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [
            [loanCoin, "loanCoin"],
            [collateralCoin, "collateralCoin"],
            [amount, "amount"],
            [direction, "direction"],
        ]
    )

    payload = {
        "loanCoin": loanCoin,
        "collateralCoin": collateralCoin,
        "amount": amount,
        "direction": direction,
        **kwargs,
    }

    return self.sign_request("GET", "/sapi/v2/futures/loan/calcAdjustLevel", payload)


def futures_loan_calc_max_adjust_amount(
    self, loanCoin: str, collateralCoin: str, **kwargs
):
    """Get Max Amount for Adjust Cross-Collateral LTV (USER_DATA)

    GET /sapi/v2/futures/loan/calcMaxAdjustAmount

    https://binance-docs.github.io/apidocs/spot/en/#get-max-amount-for-adjust-cross-collateral-ltv-v2-user_data

    Args:
        loanCoin (str)
        collateralCoin (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [[loanCoin, "loanCoin"], [collateralCoin, "collateralCoin"]]
    )

    payload = {"loanCoin": loanCoin, "collateralCoin": collateralCoin, **kwargs}

    return self.sign_request(
        "GET", "/sapi/v2/futures/loan/calcMaxAdjustAmount", payload
    )


def futures_loan_adjust_collateral(
    self, loanCoin: str, collateralCoin: str, amount: float, direction: str, **kwargs
):
    """Adjust Cross-Collateral LTV (TRADE)

    POST /sapi/v2/futures/loan/adjustCollateral

    https://binance-docs.github.io/apidocs/spot/en/#adjust-cross-collateral-ltv-v2-trade

    Args:
        loanCoin (str)
        collateralCoin (str)
        amount (float)
        direction (str): "ADDITIONAL", "REDUCED"
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [
            [loanCoin, "loanCoin"],
            [collateralCoin, "collateralCoin"],
            [amount, "amount"],
            [direction, "direction"],
        ]
    )

    payload = {
        "loanCoin": loanCoin,
        "collateralCoin": collateralCoin,
        "amount": amount,
        "direction": direction,
        **kwargs,
    }

    return self.sign_request("POST", "/sapi/v2/futures/loan/adjustCollateral", payload)


def futures_loan_adjust_collateral_history(self, **kwargs):
    """Adjust Cross-Collateral LTV History (USER_DATA)

    GET /sapi/v1/futures/loan/adjustCollateral/history

    https://binance-docs.github.io/apidocs/spot/en/#adjust-cross-collateral-ltv-history-user_data

    Keyword Args:
        loanCoin (str, optional)
        collateralCoin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request(
        "GET", "/sapi/v1/futures/loan/adjustCollateral/history", kwargs
    )


def futures_loan_liquidation_history(self, **kwargs):
    """Cross-Collateral Liquidation History (USER_DATA)

    GET /sapi/v1/futures/loan/liquidationHistory

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-liquidation-history-user_data

    Keyword Args:
        loanCoin (str, optional)
        collateralCoin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/futures/loan/liquidationHistory", kwargs)


def futures_loan_collateral_repay_limit(self, coin: str, collateralCoin: str, **kwargs):
    """Check Collateral Repay Limit (USER_DATA)

    GET /sapi/v1/futures/loan/collateralRepayLimit

    https://binance-docs.github.io/apidocs/spot/en/#check-collateral-repay-limit-user_data

    Args:
        coin (str)
        collateralCoin (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[coin, "coin"], [collateralCoin, "collateralCoin"]])

    payload = {"coin": coin, "collateralCoin": collateralCoin, **kwargs}

    return self.sign_request(
        "GET", "/sapi/v1/futures/loan/collateralRepayLimit", payload
    )


def futures_loan_collateral_repay_quote(
    self, coin: str, collateralCoin: str, amount, **kwargs
):
    """Get Collateral Repay Quote (USER_DATA)

    GET /sapi/v1/futures/loan/collateralRepay

    https://binance-docs.github.io/apidocs/spot/en/#get-collateral-repay-quote-user_data

    Args:
        coin (str)
        collateralCoin (str)
        amount (float): repay amount
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [[coin, "coin"], [collateralCoin, "collateralCoin"], [amount, "amount"]]
    )

    payload = {
        "coin": coin,
        "collateralCoin": collateralCoin,
        "amount": amount,
        **kwargs,
    }

    return self.sign_request("GET", "/sapi/v1/futures/loan/collateralRepay", payload)


def futures_loan_collateral_repay(self, quoteId: str, **kwargs):
    """Repay with Collateral (USER_DATA)

    POST /sapi/v1/futures/loan/collateralRepay

    https://binance-docs.github.io/apidocs/spot/en/#repay-with-collateral-user_data

    Args:
        quoteId (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[quoteId, "quoteId"]])

    payload = {"quoteId": quoteId, **kwargs}

    return self.sign_request("POST", "/sapi/v1/futures/loan/collateralRepay", payload)


def futures_loan_collateral_repay_result(self, quoteId: str, **kwargs):
    """Collateral Repayment Result (USER_DATA)

    GET /sapi/v1/futures/loan/collateralRepayResult

    https://binance-docs.github.io/apidocs/spot/en/#collateral-repayment-result-user_data

    Args:
        quoteId (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[quoteId, "quoteId"]])

    payload = {"quoteId": quoteId, **kwargs}

    return self.sign_request(
        "GET", "/sapi/v1/futures/loan/collateralRepayResult", payload
    )


def futures_loan_interest_history(self, **kwargs):
    """Cross-Collateral Interest History (USER_DATA)

    GET /sapi/v1/futures/loan/interestHistory

    https://binance-docs.github.io/apidocs/spot/en/#cross-collateral-interest-history-user_data

    Keyword Args:
        collateralCoin (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        limit (int, optional): default 500, max 1000
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/futures/loan/interestHistory", kwargs)



from binance.lib.utils import check_required_parameter
from binance.lib.utils import check_required_parameters


def margin_transfer(self, asset: str, amount, type: int, **kwargs):
    """Margin Account Transfer (MARGIN)
    Execute transfer between spot account and margin account.

    POST /sapi/v1/margin/transfer

    https://binance-docs.github.io/apidocs/spot/en/#margin-account-transfer-margin

    Args:
        asset (str): The asset being transferred, e.g., BTC.
        amount (float): The amount to be transferred
        type (int): 1: transfer from main account to cross margin account
                    2: transfer from cross margin account to main account
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[asset, "asset"], [amount, "amount"], [type, "type"]])

    payload = {"asset": asset, "amount": amount, "type": type, **kwargs}
    return self.sign_request("POST", "/sapi/v1/margin/transfer", payload)


def margin_borrow(self, asset: str, amount, **kwargs):
    """Margin Account Borrow (MARGIN)
    Apply for a loan.

    POST /sapi/v1/margin/load

    https://binance-docs.github.io/apidocs/spot/en/#margin-account-borrow-margin

    Args:
        asset (str): The asset being transferred, e.g., BTC.
        amount (float): The amount to be transferred
    Keyword Args:
        isIsolated (str, optional): for isolated margin or not,"TRUE", "FALSE"，default "FALSE".
        symbol (str, optional): isolated symbol
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[asset, "asset"], [amount, "amount"]])

    payload = {"asset": asset, "amount": amount, **kwargs}
    return self.sign_request("POST", "/sapi/v1/margin/loan", payload)


def margin_repay(self, asset: str, amount, **kwargs):
    """Margin Account Repay(MARGIN)
    Repay loan for margin account.

    POST /sapi/v1/margin/repay

    https://binance-docs.github.io/apidocs/spot/en/#margin-account-repay-margin

    Args:
        asset (str): The asset being transferred, e.g., BTC.
        amount (float): The amount to be transferred
    Keyword Args:
        isIsolated (str, optional): for isolated margin or not,"TRUE", "FALSE"，default "FALSE".
        symbol (str, optional): isolated symbol
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[asset, "asset"], [amount, "amount"]])

    payload = {"asset": asset, "amount": amount, **kwargs}
    return self.sign_request("POST", "/sapi/v1/margin/repay", payload)


def margin_asset(self, asset: str):
    """Query Margin Asset (MARKET_DATA)

    GET /sapi/v1/margin/asset

    https://binance-docs.github.io/apidocs/spot/en/#query-margin-asset-market_data

    Args:
        asset (str): The asset being transferred, e.g., BTC.

    """

    check_required_parameter(asset, "asset")

    payload = {"asset": asset}
    return self.limit_request("GET", "/sapi/v1/margin/asset", payload)


def margin_pair(self, symbol: str):
    """Query Margin Pair (MARKET_DATA)

    GET /sapi/v1/margin/pair

    https://binance-docs.github.io/apidocs/spot/en/#query-margin-pair-market_data

    Args:
        symbol (str)
    """

    check_required_parameter(symbol, "symbol")

    payload = {"symbol": symbol}
    return self.limit_request("GET", "/sapi/v1/margin/pair", payload)


def margin_all_assets(self):
    """Get All Margin Assets (MARKET_DATA)

    GET /sapi/v1/margin/allAssets

    https://binance-docs.github.io/apidocs/spot/en/#get-all-margin-assets-market_data

    """

    return self.limit_request("GET", "/sapi/v1/margin/allAssets")


def margin_all_pairs(self):
    """Get All Margin Pairs (MARKET_DATA)

    GET /sapi/v1/margin/allPairs

    https://binance-docs.github.io/apidocs/spot/en/#get-all-margin-pairs-market_data

    """

    return self.limit_request("GET", "/sapi/v1/margin/allPairs")


def margin_pair_index(self, symbol: str, **kwargs):
    """Query Margin PriceIndex (MARKET_DATA)

    GET /sapi/v1/margin/priceIndex

    https://binance-docs.github.io/apidocs/spot/en/#query-margin-priceindex-market_data

    Args:
        symbol (str)
    """

    check_required_parameter(symbol, "symbol")
    payload = {"symbol": symbol, **kwargs}
    return self.limit_request("GET", "/sapi/v1/margin/priceIndex", payload)


def new_margin_order(self, symbol: str, side: str, type: str, **kwargs):
    """Margin Account New Order (TRADE)

    Post a new order for margin account.

    POST /sapi/v1/margin/order

    https://binance-docs.github.io/apidocs/spot/en/#margin-account-new-order-trade

    Args:
        symbol (str)
        side (str): BUY or SELL
        type (str)
    Keyword Args:
        quantity (float, optional)
        quoteOrderQty (float, optional)
        price (float, optional)
        stopPrice (float, optional): Used with STOP_LOSS,STOP_LOSS_LIMIT,TAKE_PROFIT and TAKE_PROFIT_LIMIT orders.
        newClientOrderId (str, optional): A unique id among open orders. Automatically generated if not sent.
        icebergQty (float, optional): Used with LIMIT, STOP_LOSS_LIMIT and TAKE_PROFIT_LIMIT to create an iceberg order.
        newOrderRespType (str, optional): Set the response JSON. ACK, RESULT or FULL;
                MARKET and LIMIT order types default to FULL, all other orders default to ACK.
        sideEffectType (str, optional): NO_SIDE_EFFECT, MARGIN_BUY, AUTO_REPAY; default NO_SIDE_EFFECT.
        timeInForce (str, optional): GTC,IOC,FOK
        isIsolated (str, optional): for isolated margin or not,"TRUE", "FALSE"，default "FALSE".
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [
            [symbol, "symbol"],
            [side, "side"],
            [type, "type"],
        ]
    )

    payload = {"symbol": symbol, "side": side, "type": type, **kwargs}
    return self.sign_request("POST", "/sapi/v1/margin/order", payload)


def cancel_margin_order(self, symbol: str, **kwargs):
    """Margin Account Cancel Order (TRADE)

     Cancel an active order for margin account.

    DELETE /sapi/v1/margin/order

    https://binance-docs.github.io/apidocs/spot/en/#margin-account-cancel-order-trade

    Args:
        symbol (str)
    Keyword Args:
        orderId (int, optional)
        origClientOrderId (str, optional)
        newClientOrderId (str, optional): Used to uniquely identify this cancel. Automatically generated by default.
        isIsolated (str, optional): for isolated margin or not,"TRUE", "FALSE"，default "FALSE".
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(symbol, "symbol")
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("DELETE", "/sapi/v1/margin/order", payload)


def margin_transfer_history(self, asset: str, **kwargs):
    """Get Transfer History (USER_DATA)

    GET /sapi/v1/margin/transfer

    https://binance-docs.github.io/apidocs/spot/en/#get-transfer-history-user_data

    Args:
        asset (str)
    Keyword Args:
        type (str, optional): Transfer Type: ROLL_IN, ROLL_OUT
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10 Max:100
        archived (str, optional): Default: false. Set to true for archived data from 6 months ago
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(asset, "asset")
    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/margin/transfer", payload)


def margin_load_record(self, asset: str, **kwargs):
    """Query Loan Record (USER_DATA)

    GET /sapi/v1/margin/loan

    https://binance-docs.github.io/apidocs/spot/en/#query-loan-record-user_data

    Args:
        asset (str)
    Keyword Args:
        isolatedSymbol (str, optional): isolated symbol
        txId (int, optional): the tranId in POST /sapi/v1/margin/loan
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10 Max:100
        archived (str, optional): Default: false. Set to true for archived data from 6 months ago
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")
    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/margin/loan", payload)


def margin_repay_record(self, asset: str, **kwargs):
    """Query Repay Record (USER_DATA)

    GET /sapi/v1/margin/repay

    https://binance-docs.github.io/apidocs/spot/en/#query-repay-record-user_data

    Args:
        asset (str)
    Keyword Args:
        isolatedSymbol (str, optional): isolated symbol
        txId (int, optional): return of /sapi/v1/margin/repay
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10 Max:100
        archived (str, optional): Default: false. Set to true for archived data from 6 months ago
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")
    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/margin/repay", payload)


def margin_interest_history(self, **kwargs):
    """Get Interest History (USER_DATA)

    GET /sapi/v1/margin/interestHistory

    https://binance-docs.github.io/apidocs/spot/en/#get-interest-history-user_data

    Keyword Args:
        asset (str, optional)
        isolatedSymbol (str, optional): isolated symbol
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10 Max:100
        archived (str, optional): Default: false. Set to true for archived data from 6 months ago
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/margin/interestHistory", kwargs)


def margin_force_liquidation_record(self, **kwargs):
    """Get Force Liquidation Record (USER_DATA)

    GET /sapi/v1/margin/forceLiquidationRec

    https://binance-docs.github.io/apidocs/spot/en/#get-force-liquidation-record-user_data

    Keyword Args:
        isolatedSymbol (str, optional): isolated symbol
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10 Max:100
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/margin/forceLiquidationRec", kwargs)


def margin_account(self, **kwargs):
    """Query Cross Margin Account Details (USER_DATA)

    GET /sapi/v1/margin/account

    https://binance-docs.github.io/apidocs/spot/en/#query-margin-account-details-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/margin/account", kwargs)


def margin_order(self, symbol: str, **kwargs):
    """Query Margin Account's Order (USER_DATA)

    GET /sapi/v1/margin/order

    https://binance-docs.github.io/apidocs/spot/en/#query-margin-account-39-s-order-user_data

    Args:
        symbol (str)
    Keyword Args:
        orderId (str, optional)
        origClientOrderId (str, optional)
        isIsolated (str, optional): for isolated margin or not,"TRUE", "FALSE"，default "FALSE".
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(symbol, "symbol")
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("GET", "/sapi/v1/margin/order", payload)


def margin_open_orders(self, **kwargs):
    """Query Margin Account's Open Order (USER_DATA)

    GET /sapi/v1/margin/openOrders

    https://binance-docs.github.io/apidocs/spot/en/#query-margin-account-39-s-open-order-user_data

    Keyword Args:
        symbol (str, optional)
        isIsolated (str, optional): for isolated margin or not,"TRUE", "FALSE"，default "FALSE".
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/margin/openOrders", kwargs)


def margin_open_orders_cancellation(self, symbol: str, **kwargs):
    """Margin Account Cancel all Open Orders on a Symbol (USER_DATA)

    DELETE /sapi/v1/margin/openOrders

    https://binance-docs.github.io/apidocs/spot/en/#margin-account-cancel-all-open-orders-on-a-symbol-trade

    Args:
        symbol (str)
    Keyword Args:
        isIsolated (str, optional): for isolated margin or not,"TRUE", "FALSE"，default "FALSE".
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(symbol, "symbol")
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("DELETE", "/sapi/v1/margin/openOrders", payload)


def margin_all_orders(self, symbol: str, **kwargs):
    """Query Margin Account's All Orders (USER_DATA)

    GET /sapi/v1/margin/allOrders

    https://binance-docs.github.io/apidocs/spot/en/#query-margin-account-39-s-all-order-user_data

    Args:
        symbol (str)
    Keyword Args:
        orderId (int, optional)
        isIsolated (str, optional): for isolated margin or not,"TRUE", "FALSE"，default "FALSE".
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): Default 500; max 500.
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(symbol, "symbol")
    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("GET", "/sapi/v1/margin/allOrders", payload)


def margin_my_trades(self, symbol: str, **kwargs):
    """Query Margin Account's Trade List (USER_DATA)

    GET /sapi/v1/margin/myTrades

    https://binance-docs.github.io/apidocs/spot/en/#query-margin-account-39-s-trade-list-user_data

    Args:
        symbol (str)
    Keyword Args:
        fromID (int, optional): TradeId to fetch from. Default gets most recent trades.
        isIsolated (str, optional): for isolated margin or not,"TRUE", "FALSE"，default "FALSE".
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): Default 500; max 500.
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(symbol, "symbol")

    payload = {"symbol": symbol, **kwargs}

    return self.sign_request("GET", "/sapi/v1/margin/myTrades", payload)


def margin_max_borrowable(self, asset: str, **kwargs):
    """Query Max Borrow (USER_DATA)

    GET /sapi/v1/margin/maxBorrowable

    https://binance-docs.github.io/apidocs/spot/en/#query-max-borrow-user_data

    Args:
        asset (str)
    Keyword Args:
        isolatedSymbol (str, optional): isolated symbol
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")
    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/margin/maxBorrowable", payload)


def margin_max_transferable(self, asset: str, **kwargs):
    """Query Max Transfer-Out Amount (USER_DATA)

    GET /sapi/v1/margin/maxTransferable

    https://binance-docs.github.io/apidocs/spot/en/#query-max-transfer-out-amount-user_data

    Args:
        asset (str)
    Keyword Args:
        isolatedSymbol (str, optional): isolated symbol
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")
    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/margin/maxBorrowable", payload)


def isolated_margin_transfer(
    self, asset: str, symbol: str, transFrom: str, transTo: str, amount, **kwargs
):
    """Isolated Margin Account Transfer (MARGIN)

    POST /sapi/v1/margin/isolated/transfer

    https://binance-docs.github.io/apidocs/spot/en/#isolated-margin-account-transfer-margin

    Args:
        asset (str)
        symbol (str)
        amount (float)
        transFrom (str): "SPOT", "ISOLATED_MARGIN"
        transTo (str): "SPOT", "ISOLATED_MARGIN"
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [
            [asset, "asset"],
            [symbol, "symbol"],
            [transFrom, "transFrom"],
            [transTo, "transTo"],
            [amount, "amount"],
        ]
    )

    payload = {
        "asset": asset,
        "symbol": symbol,
        "transFrom": transFrom,
        "transTo": transTo,
        "amount": amount,
        **kwargs,
    }
    return self.sign_request("POST", "/sapi/v1/margin/isolated/transfer", payload)


def isolated_margin_transfer_history(self, symbol: str, **kwargs):
    """Get Isolated Margin Transfer History (USER_DATA)

    GET /sapi/v1/margin/isolated/transfer

    https://binance-docs.github.io/apidocs/spot/en/#get-isolated-margin-transfer-history-user_data

    Args:
        symbol (str)
    Keyword Args:
        asset (str, optional): asset,such as BTC
        transFrom (str, optional): "SPOT", "ISOLATED_MARGIN"
        transTo (str, optional): "SPOT", "ISOLATED_MARGIN"
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10 Max:100
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(symbol, "symbol")

    payload = {"symbol": symbol, **kwargs}
    return self.sign_request("GET", "/sapi/v1/margin/isolated/transfer", payload)


def isolated_margin_account(self, **kwargs):
    """Query Isolated Margin Account Info (USER_DATA)

    GET /sapi/v1/margin/isolated/account

    https://binance-docs.github.io/apidocs/spot/en/#query-isolated-margin-account-info-user_data

    Keyword Args:
        symbols (str, optional): Max 5 symbols can be sent; separated by ",". e.g. "BTCUSDT,BNBUSDT,ADAUSDT"
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/margin/isolated/account", kwargs)


def isolated_margin_pair(self, symbol: str, **kwargs):
    """Query Isolated Margin Symbol (USER_DATA)

    GET /sapi/v1/margin/isolated/pair

    https://binance-docs.github.io/apidocs/spot/en/#query-isolated-margin-account-info-user_data

    Args:
        symbol (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(symbol, "symbol")

    payload = {"symbol": symbol, **kwargs}

    return self.sign_request("GET", "/sapi/v1/margin/isolated/pair", payload)


def isolated_margin_all_pairs(self, **kwargs):
    """Get All Isolated Margin Symbol(USER_DATA)

    GET /sapi/v1/margin/isolated/allPairs

    https://binance-docs.github.io/apidocs/spot/en/#get-all-isolated-margin-symbol-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/margin/isolated/allPairs", kwargs)


def toggle_bnbBurn(self, **kwargs):
    """Toggle BNB Burn On Spot Trade And Margin Interest (USER_DATA)

    POST /sapi/v1/bnbBurn

    https://binance-docs.github.io/apidocs/spot/en/#toggle-bnb-burn-on-spot-trade-and-margin-interest-user_data

    Keyword Args:
        spotBNBBurn (str, optional): "true" or "false"; Determines whether to use BNB to pay for trading fees on SPOT
        interestBNBBurn (str, optional): "true" or "false"; Determines whether to use BNB to pay for margin loan's interest
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("POST", "/sapi/v1/bnbBurn", kwargs)


def bnbBurn_status(self, **kwargs):
    """Get BNB Burn Status (USER_DATA)

    GET /sapi/v1/bnbBurn

    https://binance-docs.github.io/apidocs/spot/en/#get-bnb-burn-status-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/bnbBurn", kwargs)


def margin_interest_rate_history(self, asset: str, **kwargs):
    """Get Margin Interest Rate History (USER_DATA)

    GET /sapi/v1/margin/interestRateHistory

    https://binance-docs.github.io/apidocs/spot/en/#query-margin-interest-rate-history-user_data

    Args:
        asset (str)
    Keyword Args:
        vipLevel (str, optional): Default: user's vip level
        startTime (int, optional): Default: 7 days ago.
        endTime (int, optional): Default: present. Maximum range: 3 months.
        limit (int, optional): Default: 20. Maximum: 100.
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")
    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/margin/interestRateHistory", payload)



import logging
from autobahn.twisted.websocket import WebSocketClientFactory
from twisted.internet.protocol import ReconnectingClientFactory
from binance.websocket.binance_client_protocol import BinanceClientProtocol


class BinanceReconnectingClientFactory(ReconnectingClientFactory):

    initialDelay = 0.1
    maxDelay = 10
    maxRetries = 10


class BinanceClientFactory(WebSocketClientFactory, BinanceReconnectingClientFactory):
    def __init__(self, *args, payload=None, **kwargs):
        WebSocketClientFactory.__init__(self, *args, **kwargs)
        self.protocol_instance = None
        self.base_client = None
        self.payload = payload

    _reconnect_error_payload = {"e": "error", "m": "Max reconnect retries reached"}

    def startedConnecting(self, connector):
        logging.info("Start to connect....")

    def clientConnectionFailed(self, connector, reason):
        logging.error(
            "Can't connect to server. Reason: {}. Retrying: {}".format(
                reason, self.retries + 1
            )
        )
        self.retry(connector)
        if self.retries > self.maxRetries:
            self.callback(self._reconnect_error_payload)

    def clientConnectionLost(self, connector, reason):
        logging.error(
            "Lost connection to Server. Reason: {}. Retrying: {}".format(
                reason, self.retries + 1
            )
        )
        self.retry(connector)
        if self.retries > self.maxRetries:
            self.callback(self._reconnect_error_payload)

    def buildProtocol(self, addr):
        return BinanceClientProtocol(self, payload=self.payload)



import json
from twisted.internet import reactor
from binance.websocket.websocket_client import BinanceWebsocketClient


class SpotWebsocketClient(BinanceWebsocketClient):
    def __init__(self, stream_url="wss://stream.binance.com:9443"):
        super().__init__(stream_url)

    def agg_trade(self, symbol: str, id: int, callback, **kwargs):
        """Aggregate Trade Streams

        The Aggregate Trade Streams push trade information that is aggregated for a single taker order.

        Stream Name: <symbol>@aggTrade

        Update Speed: Real-time
        """
        self.live_subscribe(
            "{}@aggTrade".format(symbol.lower()), id, callback, **kwargs
        )

    def trade(self, symbol: str, id: int, callback, **kwargs):
        """Trade Streams

        The Trade Streams push raw trade information; each trade has a unique buyer and seller.

        Stream Name: <symbol>@trade

        Update Speed: Real-time
        """
        self.live_subscribe("{}@trade".format(symbol.lower()), id, callback, **kwargs)

    def kline(self, symbol: str, id: int, interval: str, callback, **kwargs):
        """Kline/Candlestick Streams

        The Kline/Candlestick Stream push updates to the current klines/candlestick every second.

        Stream Name: <symbol>@kline_<interval>

        interval:
        m -> minutes; h -> hours; d -> days; w -> weeks; M -> months

        - 1m
        - 3m
        - 5m
        - 15m
        - 30m
        - 1h
        - 2h
        - 4h
        - 6h
        - 8h
        - 12h
        - 1d
        - 3d
        - 1w
        - 1M

        Update Speed: 2000ms
        """

        self.live_subscribe(
            "{}@kline_{}".format(symbol.lower(), interval), id, callback, **kwargs
        )

    def mini_ticker(self, id: int, callback, symbol=None, **kwargs):
        """Individual symbol or all symbols mini ticker

        24hr rolling window mini-ticker statistics.
        These are NOT the statistics of the UTC day, but a 24hr rolling window for the previous 24hrs

        Stream Name: <symbol>@miniTicker or
        Stream Name: !miniTicker@arr

        Update Speed: 1000ms
        """

        if symbol is None:
            self.live_subscribe("!miniTicker@arr", id, callback, **kwargs)
        else:
            self.live_subscribe(
                "{}@miniTicker".format(symbol.lower()), id, callback, **kwargs
            )

    def ticker(self, id: int, callback, symbol=None, **kwargs):
        """Individual symbol or all symbols ticker

        24hr rolling window ticker statistics for a single symbol.
        These are NOT the statistics of the UTC day, but a 24hr rolling window for the previous 24hrs.

        Stream Name: <symbol>@ticker or
        Stream Name: !ticker@arr

        Update Speed: 1000ms
        """

        if symbol is None:
            self.live_subscribe("!ticker@arr", id, callback, **kwargs)
        else:
            self.live_subscribe(
                "{}@ticker".format(symbol.lower()), id, callback, **kwargs
            )

    def book_ticker(self, id: int, callback, symbol=None, **kwargs):
        """Individual symbol or all book ticker

        Pushes any update to the best bid or ask's price or quantity in real-time for a specified symbol.

        Stream Name: <symbol>@bookTicker or
        Stream Name: !bookTicker

        Update Speed: realtime
        """

        if symbol is None:
            self.live_subscribe("!bookTicker", id, callback, **kwargs)
        else:
            self.live_subscribe(
                "{}@bookTicker".format(symbol.lower()), id, callback, **kwargs
            )

    def partial_book_depth(
        self, symbol: str, id: int, level, speed, callback, **kwargs
    ):
        """Partial Book Depth Streams

        Top bids and asks, Valid are 5, 10, or 20.

        Stream Names: <symbol>@depth<levels> OR <symbol>@depth<levels>@100ms.

        Update Speed: 1000ms or 100ms
        """
        self.live_subscribe(
            "{}@depth{}@{}ms".format(symbol.lower(), level, speed),
            id,
            callback,
            **kwargs
        )

    def diff_book_depth(self, symbol: str, id: int, speed, callback, **kwargs):
        """Diff. Depth Stream

        Stream Name: <symbol>@depth OR <symbol>@depth@100ms

        Update Speed: 1000ms or 100ms

        Order book price and quantity depth updates used to locally manage an order book.
        """
        self.live_subscribe(
            "{}@depth@{}ms".format(symbol.lower(), speed), id, callback, **kwargs
        )

    def user_data(self, listen_key: str, id: int, callback, **kwargs):
        """listen to user data by provided listenkey"""
        self.live_subscribe(listen_key, id, callback, **kwargs)



import json
import logging
import threading
from urllib.parse import urlparse
from twisted.internet import reactor, ssl
from twisted.internet.error import ReactorAlreadyRunning
from autobahn.twisted.websocket import WebSocketClientFactory, connectWS
from binance.websocket.binance_client_protocol import BinanceClientProtocol
from binance.websocket.binance_client_factory import BinanceClientFactory


class BinanceSocketManager(threading.Thread):
    def __init__(self, stream_url):
        threading.Thread.__init__(self)

        self.factories = {}
        self._connected_event = threading.Event()
        self.stream_url = stream_url
        self._conns = {}
        self._user_callback = None

    def _start_socket(
        self, stream_name, payload, callback, is_combined=False, is_live=True
    ):
        if stream_name in self._conns:
            return False

        if is_combined:
            factory_url = self.stream_url + "/stream"
        else:
            factory_url = self.stream_url + "/ws"

        if not is_live:
            payload_obj = json.loads(payload.decode("utf8"))

            if is_combined:
                factory_url = factory_url + "?streams=" + payload_obj["params"]
            else:
                factory_url = factory_url + "/" + payload_obj["params"]
            payload = None

        logging.info("Connection with URL: {}".format(factory_url))

        factory = BinanceClientFactory(factory_url, payload=payload)
        factory.base_client = self
        factory.protocol = BinanceClientProtocol
        factory.setProtocolOptions(
            openHandshakeTimeout=5, autoPingInterval=300, autoPingTimeout=5
        )
        factory.callback = callback
        self.factories[stream_name] = factory
        reactor.callFromThread(self.add_connection, stream_name, self.stream_url)

    def add_connection(self, stream_name, url):
        if not url.startswith("wss://"):
            raise ValueError("expected wss:// URL prefix")

        factory = self.factories[stream_name]
        options = ssl.optionsForClientTLS(hostname=urlparse(url).hostname)
        self._conns[stream_name] = connectWS(factory, options)

    def stop_socket(self, conn_key):
        if conn_key not in self._conns:
            return

        # disable reconnecting if we are closing
        self._conns[conn_key].factory = WebSocketClientFactory(self.stream_url)
        self._conns[conn_key].disconnect()
        del self._conns[conn_key]

    def run(self):
        try:
            reactor.run(installSignalHandlers=False)
        except ReactorAlreadyRunning:
            # Ignore error about reactor already running
            pass

    def close(self):
        keys = set(self._conns.keys())
        for key in keys:
            self.stop_socket(key)
        self._conns = {}



from binance.lib.utils import check_required_parameters


def bswap_pools(self):
    """List All Swap Pools (MARKET_DATA)
    Get metadata about all swap pools.

    GET /sapi/v1/bswap/pools

    https://binance-docs.github.io/apidocs/spot/en/#list-all-swap-pools-market_data

    """

    return self.limit_request("GET", "/sapi/v1/bswap/pools", {})


def bswap_liquidity(self, **kwargs):
    """Get liquidity information of a pool (USER_DATA)
    Get liquidity information and user share of a pool

    GET /sapi/v1/bswap/liquidity

    https://binance-docs.github.io/apidocs/spot/en/#get-liquidity-information-of-a-pool-user_data

    Keyword Args:
        poolId (int, optional)
        recvWindow (int, optional): The value cannot be greater than 60000

    """
    return self.sign_request("GET", "/sapi/v1/bswap/liquidity", kwargs)


def bswap_liquidity_add(self, poolId: int, asset: str, quantity: float, **kwargs):
    """Add Liquidity (TRADE)
    Add liquidity to a pool.

    POST /sapi/v1/bswap/liquidityAdd

    https://binance-docs.github.io/apidocs/spot/en/#add-liquidity-trade

    Args:
        poolId (int)
        asset (str)
        quantity (float)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters(
        [[poolId, "poolId"], [asset, "asset"], [quantity, "quantity"]]
    )
    payload = {"poolId": poolId, "asset": asset, "quantity": quantity, **kwargs}

    return self.sign_request("POST", "/sapi/v1/bswap/liquidityAdd", payload)


def bswap_liquidity_remove(
    self, poolId: str, type: str, asset: list, shareAmount, **kwargs
):
    """Remove Liquidity (TRADE)
    Remove liquidity from a pool, type include SINGLE and COMBINATION, asset is mandatory for single asset removal

    POST /sapi/v1/bswap/liquidityRemove

    https://binance-docs.github.io/apidocs/spot/en/#remove-liquidity-trade

    Args:
        poolId (int)
        type (str): SINGLE for single asset removal, COMBINATION for combination of all coins removal
        asset (str)
        shareAmount (float): Mandatory for liquidity removal
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters(
        [
            [poolId, "poolId"],
            [type, "type"],
            [asset, "asset"],
            [shareAmount, "shareAmount"],
        ]
    )
    payload = {
        "poolId": poolId,
        "type": type,
        "asset": ",".join(asset),
        "shareAmount": shareAmount,
        **kwargs,
    }

    return self.sign_request("POST", "/sapi/v1/bswap/liquidityRemove", payload)


def bswap_liquidity_operation_record(self, **kwargs):
    """Get Liquidity Operation Record (USER_DATA)
    Get liquidity operation (add/remove) records.

    GET /sapi/v1/bswap/liquidityOps

    https://binance-docs.github.io/apidocs/spot/en/#get-liquidity-operation-record-user_data

    Keyword Args:
        operationId (int, optional)
        poolId (int, optional)
        operation (str, optional): ADD or REMOVE
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): default 3, max 100
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    return self.sign_request("GET", "/sapi/v1/bswap/liquidityOps", kwargs)


def bswap_request_quote(
    self, quoteAsset: str, baseAsset: str, quoteQty: float, **kwargs
):
    """Request Quote (USER_DATA)
    Request a quote for swap quote asset (selling asset) for base asset (buying asset), essentially price/exchange rates.
    quoteQty is quantity of quote asset (to sell).

    Please be noted the quote is for reference only, the actual price will change as the liquidity changes,
    it's recommended to swap immediate after request a quote for slippage prevention.

    GET /sapi/v1/bswap/quote

    https://binance-docs.github.io/apidocs/spot/en/#request-quote-user_data

    Args:
        quoteAsset (str)
        baseAsset (str)
        quoteQty (float)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [[quoteAsset, "quoteAsset"], [baseAsset, "baseAsset"], [quoteQty, "quoteQty"]]
    )
    payload = {
        "quoteAsset": quoteAsset,
        "baseAsset": baseAsset,
        "quoteQty": quoteQty,
        **kwargs,
    }

    return self.sign_request("GET", "/sapi/v1/bswap/quote", payload)


def bswap_swap(self, quoteAsset: str, baseAsset: str, quoteQty: float, **kwargs):
    """Swap (TRADE)
    Swap quoteAsset for baseAsset.

    POST /sapi/v1/bswap/swap

    https://binance-docs.github.io/apidocs/spot/en/#swap-trade

    Args:
        baseAsset (str)
        quoteAsset (str)
        quoteQty (float)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters(
        [[quoteAsset, "quoteAsset"], [baseAsset, "baseAsset"], [quoteQty, "quoteQty"]]
    )
    payload = {
        "quoteAsset": quoteAsset,
        "baseAsset": baseAsset,
        "quoteQty": quoteQty,
        **kwargs,
    }
    return self.sign_request("POST", "/sapi/v1/bswap/swap", payload)


def bswap_swap_history(self, **kwargs):
    """Get Swap History (USER_DATA)
    Get swap history.

    GET /sapi/v1/bswap/swap

    https://binance-docs.github.io/apidocs/spot/en/#get-swap-history-user_data

    Keyword Args:
        swapId (int, optional)
        startTime (int, optional)
        endTime (int, optional)
        status (int, optional)
        baseAsset (str, optional)
        quoteAsset (str, optional)
        limit (int, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    return self.sign_request("GET", "/sapi/v1/bswap/swap", kwargs)



from binance.lib.utils import check_required_parameter
from binance.lib.utils import check_required_parameters


def ping(self):
    """Test Connectivity
    Test connectivity to the Rest API.

    GET /api/v3/ping

    https://binance-docs.github.io/apidocs/spot/en/#test-connectivity

    """

    url_path = "/api/v3/ping"
    return self.query(url_path)


def time(self):
    """Check Server Time
    Test connectivity to the Rest API and get the current server time.

    GET /api/v3/time

    https://binance-docs.github.io/apidocs/spot/en/#check-server-time

    """

    url_path = "/api/v3/time"
    return self.query(url_path)


def exchange_info(self):
    """Exchange Information
    Current exchange trading rules and symbol information

    GET /api/v3/exchangeinfo

    https://binance-docs.github.io/apidocs/spot/en/#exchange-information

    """

    url_path = "/api/v3/exchangeInfo"
    return self.query(url_path)


def depth(self, symbol: str, **kwargs):
    """Get orderbook.

    GET /api/v3/depth

    https://binance-docs.github.io/apidocs/spot/en/#order-book

    Args:
        symbol (str): the trading pair
    Keyword Args:
        limit (int, optional): limit the results. Default 100; valid limits:[5, 10, 20, 50, 100, 500, 1000, 5000]
    """

    check_required_parameter(symbol, "symbol")
    params = {"symbol": symbol, **kwargs}
    return self.query("/api/v3/depth", params)


def trades(self, symbol: str, **kwargs):
    """Recent Trades List
    Get recent trades (up to last 500).

    GET /api/v3/trades

    https://binance-docs.github.io/apidocs/spot/en/#recent-trades-list

    Args:
        symbol (str): the trading pair
    Keyword Args:
        limit (int, optional): limit the results. Default 500; max 1000.
    """
    check_required_parameter(symbol, "symbol")
    params = {"symbol": symbol, **kwargs}
    return self.query("/api/v3/trades", params)


def historical_trades(self, symbol: str, **kwargs):
    """Old Trade Lookup
    Get older market trades.

    GET /api/v3/historicalTrades

    https://binance-docs.github.io/apidocs/spot/en/#old-trade-lookup

    Args:
        symbol (str): the trading pair
    Keyword Args:
        limit (int, optional): limit the results. Default 500; max 1000.
        formId (int, optional): trade id to fetch from. Default gets most recent trades.
    """
    check_required_parameter(symbol, "symbol")
    params = {"symbol": symbol, **kwargs}
    return self.limit_request("GET", "/api/v3/historicalTrades", params)


def agg_trades(self, symbol: str, **kwargs):
    """Compressed/Aggregate Trades List

    GET /api/v3/aggTrades

    https://binance-docs.github.io/apidocs/spot/en/#compressed-aggregate-trades-list

    Args:
        symbol (str): the trading pair
    Keyword Args:
        limit (int, optional): limit the results. Default 500; max 1000.
        formId (int, optional): id to get aggregate trades from INCLUSIVE.
        startTime (int, optional): Timestamp in ms to get aggregate trades from INCLUSIVE.
        endTime (int, optional): Timestamp in ms to get aggregate trades until INCLUSIVE.
    """

    check_required_parameter(symbol, "symbol")
    params = {"symbol": symbol, **kwargs}
    return self.query("/api/v3/aggTrades", params)


def klines(self, symbol: str, interval: str, **kwargs):
    """Kline/Candlestick Data

    GET /api/v3/klines

    https://binance-docs.github.io/apidocs/spot/en/#kline-candlestick-data

    Args:
        symbol (str): the trading pair
        interval (str): the interval of kline, e.g 1m, 5m, 1h, 1d, etc.
    Keyword Args:
        limit (int, optional): limit the results. Default 500; max 1000.
        startTime (int, optional): Timestamp in ms to get aggregate trades from INCLUSIVE.
        endTime (int, optional): Timestamp in ms to get aggregate trades until INCLUSIVE.
    """
    check_required_parameters([[symbol, "symbol"], [interval, "interval"]])

    params = {"symbol": symbol, "interval": interval, **kwargs}
    return self.query("/api/v3/klines", params)


def avg_price(self, symbol: str):
    """Current Average Price

    GET /api/v3/avgPrice

    https://binance-docs.github.io/apidocs/spot/en/#current-average-price

    Args:
        symbol (str): the trading pair
    """

    check_required_parameter(symbol, "symbol")
    params = {
        "symbol": symbol,
    }
    return self.query("/api/v3/avgPrice", params)


def ticker_24hr(self, symbol: str = None):
    """24hr Ticker Price Change Statistics

    GET /api/v3/ticker/24hr

    https://binance-docs.github.io/apidocs/spot/en/#24hr-ticker-price-change-statistics

    Args:
        symbol (str, optional): the trading pair
    """

    params = {
        "symbol": symbol,
    }
    return self.query("/api/v3/ticker/24hr", params)


def ticker_price(self, symbol: str = None):
    """Symbol Price Ticker

    GET /api/v3/ticker/price

    https://binance-docs.github.io/apidocs/spot/en/#symbol-price-ticker

    Args:
        symbol (str, optional): the trading pair
    """

    params = {
        "symbol": symbol,
    }
    return self.query("/api/v3/ticker/price", params)


def book_ticker(self, symbol: str = None):
    """Symbol Order Book Ticker

    GET /api/v3/ticker/bookTicker

    https://binance-docs.github.io/apidocs/spot/en/#symbol-order-book-ticker

    Args:
        symbol (str, optional): the trading pair
    """

    params = {
        "symbol": symbol,
    }
    return self.query("/api/v3/ticker/bookTicker", params)



from binance.api import API


class Spot(API):
    def __init__(self, key=None, secret=None, **kwargs):
        if "base_url" not in kwargs:
            kwargs["base_url"] = "https://api.binance.com"
        super().__init__(key, secret, **kwargs)

    # MARKETS
    from binance.spot.market import ping
    from binance.spot.market import time
    from binance.spot.market import exchange_info
    from binance.spot.market import depth
    from binance.spot.market import trades
    from binance.spot.market import historical_trades
    from binance.spot.market import agg_trades
    from binance.spot.market import klines
    from binance.spot.market import avg_price
    from binance.spot.market import ticker_24hr
    from binance.spot.market import ticker_price
    from binance.spot.market import book_ticker

    # ACCOUNT(including orders and trades)
    from binance.spot.account import new_order_test
    from binance.spot.account import new_order
    from binance.spot.account import cancel_order
    from binance.spot.account import cancel_open_orders
    from binance.spot.account import get_order
    from binance.spot.account import get_open_orders
    from binance.spot.account import get_orders
    from binance.spot.account import new_oco_order
    from binance.spot.account import cancel_oco_order
    from binance.spot.account import get_oco_order
    from binance.spot.account import get_oco_orders
    from binance.spot.account import get_oco_open_orders
    from binance.spot.account import account
    from binance.spot.account import my_trades

    # STREAMS
    from binance.spot.data_stream import new_listen_key
    from binance.spot.data_stream import renew_listen_key
    from binance.spot.data_stream import close_listen_key
    from binance.spot.data_stream import new_margin_listen_key
    from binance.spot.data_stream import renew_margin_listen_key
    from binance.spot.data_stream import close_margin_listen_key
    from binance.spot.data_stream import new_isolated_margin_listen_key
    from binance.spot.data_stream import renew_isolated_margin_listen_key
    from binance.spot.data_stream import close_isolated_margin_listen_key

    # MARGIN
    from binance.spot.margin import margin_transfer
    from binance.spot.margin import margin_borrow
    from binance.spot.margin import margin_repay
    from binance.spot.margin import margin_asset
    from binance.spot.margin import margin_pair
    from binance.spot.margin import margin_all_assets
    from binance.spot.margin import margin_all_pairs
    from binance.spot.margin import margin_pair_index
    from binance.spot.margin import new_margin_order
    from binance.spot.margin import cancel_margin_order
    from binance.spot.margin import margin_transfer_history
    from binance.spot.margin import margin_load_record
    from binance.spot.margin import margin_repay_record
    from binance.spot.margin import margin_interest_history
    from binance.spot.margin import margin_force_liquidation_record
    from binance.spot.margin import margin_account
    from binance.spot.margin import margin_order
    from binance.spot.margin import margin_open_orders
    from binance.spot.margin import margin_open_orders_cancellation
    from binance.spot.margin import margin_all_orders
    from binance.spot.margin import margin_my_trades
    from binance.spot.margin import margin_max_borrowable
    from binance.spot.margin import margin_max_transferable
    from binance.spot.margin import isolated_margin_transfer
    from binance.spot.margin import isolated_margin_transfer_history
    from binance.spot.margin import isolated_margin_account
    from binance.spot.margin import isolated_margin_pair
    from binance.spot.margin import isolated_margin_all_pairs
    from binance.spot.margin import toggle_bnbBurn
    from binance.spot.margin import bnbBurn_status
    from binance.spot.margin import margin_interest_rate_history

    # SAVINGS
    from binance.spot.savings import savings_flexible_products
    from binance.spot.savings import savings_flexible_user_left_quota
    from binance.spot.savings import savings_purchase_flexible_product
    from binance.spot.savings import savings_flexible_user_redemption_quota
    from binance.spot.savings import savings_flexible_redeem
    from binance.spot.savings import savings_flexible_product_position
    from binance.spot.savings import savings_project_list
    from binance.spot.savings import savings_purchase_project
    from binance.spot.savings import savings_project_position
    from binance.spot.savings import savings_account
    from binance.spot.savings import savings_purchase_record
    from binance.spot.savings import savings_redemption_record
    from binance.spot.savings import savings_interest_history
    from binance.spot.savings import savings_change_position

    # WALLET
    from binance.spot.wallet import system_status
    from binance.spot.wallet import coin_info
    from binance.spot.wallet import account_snapshot
    from binance.spot.wallet import disable_fast_withdraw
    from binance.spot.wallet import enable_fast_withdraw
    from binance.spot.wallet import withdraw
    from binance.spot.wallet import deposit_history
    from binance.spot.wallet import withdraw_history
    from binance.spot.wallet import deposit_address
    from binance.spot.wallet import account_status
    from binance.spot.wallet import api_trading_status
    from binance.spot.wallet import dust_log
    from binance.spot.wallet import user_universal_transfer
    from binance.spot.wallet import user_universal_transfer_history
    from binance.spot.wallet import transfer_dust
    from binance.spot.wallet import asset_dividend_record
    from binance.spot.wallet import asset_detail
    from binance.spot.wallet import trade_fee

    # mining
    from binance.spot.mining import mining_algo_list
    from binance.spot.mining import mining_coin_list
    from binance.spot.mining import mining_worker
    from binance.spot.mining import mining_worker_list
    from binance.spot.mining import mining_earnings_list
    from binance.spot.mining import mining_bonus_list
    from binance.spot.mining import mining_statistics_list
    from binance.spot.mining import mining_account_list
    from binance.spot.mining import mining_hashrate_resale_request
    from binance.spot.mining import mining_hashrate_resale_cancellation
    from binance.spot.mining import mining_hashrate_resale_list
    from binance.spot.mining import mining_hashrate_resale_details

    # SUB-ACCOUNT
    from binance.spot.sub_account import sub_account_create
    from binance.spot.sub_account import sub_account_list
    from binance.spot.sub_account import sub_account_assets
    from binance.spot.sub_account import sub_account_deposit_address
    from binance.spot.sub_account import sub_account_deposit_history
    from binance.spot.sub_account import sub_account_status
    from binance.spot.sub_account import sub_account_enable_margin
    from binance.spot.sub_account import sub_account_margin_account
    from binance.spot.sub_account import sub_account_margin_account_summary
    from binance.spot.sub_account import sub_account_enable_futures
    from binance.spot.sub_account import sub_account_futures_transfer
    from binance.spot.sub_account import sub_account_margin_transfer
    from binance.spot.sub_account import sub_account_transfer_to_sub
    from binance.spot.sub_account import sub_account_transfer_to_master
    from binance.spot.sub_account import sub_account_transfer_sub_account_history
    from binance.spot.sub_account import sub_account_futures_asset_transfer_history
    from binance.spot.sub_account import sub_account_futures_asset_transfer
    from binance.spot.sub_account import sub_account_spot_summary
    from binance.spot.sub_account import sub_account_universal_transfer
    from binance.spot.sub_account import sub_account_universal_transfer_history
    from binance.spot.sub_account import sub_account_futures_account
    from binance.spot.sub_account import sub_account_futures_account_summary
    from binance.spot.sub_account import sub_account_futures_position_risk
    from binance.spot.sub_account import sub_account_spot_transfer_history
    from binance.spot.sub_account import sub_account_enable_leverage_token

    # Futures
    from binance.spot.futures import futures_transfer
    from binance.spot.futures import futures_transfer_history
    from binance.spot.futures import futures_loan_borrow
    from binance.spot.futures import futures_loan_borrow_history
    from binance.spot.futures import futures_loan_repay
    from binance.spot.futures import futures_loan_repay_history
    from binance.spot.futures import futures_loan_wallet
    from binance.spot.futures import futures_loan_configs
    from binance.spot.futures import futures_loan_calc_adjust_level
    from binance.spot.futures import futures_loan_calc_max_adjust_amount
    from binance.spot.futures import futures_loan_adjust_collateral
    from binance.spot.futures import futures_loan_adjust_collateral_history
    from binance.spot.futures import futures_loan_liquidation_history
    from binance.spot.futures import futures_loan_collateral_repay_limit
    from binance.spot.futures import futures_loan_collateral_repay_quote
    from binance.spot.futures import futures_loan_collateral_repay
    from binance.spot.futures import futures_loan_collateral_repay_result
    from binance.spot.futures import futures_loan_interest_history

    # BLVTs
    from binance.spot.blvt import blvt_info
    from binance.spot.blvt import subscribe_blvt
    from binance.spot.blvt import subscription_record
    from binance.spot.blvt import redeem_blvt
    from binance.spot.blvt import redemption_record
    from binance.spot.blvt import user_limit_info

    # BSwap
    from binance.spot.bswap import bswap_pools
    from binance.spot.bswap import bswap_liquidity
    from binance.spot.bswap import bswap_liquidity_add
    from binance.spot.bswap import bswap_liquidity_remove
    from binance.spot.bswap import bswap_liquidity_operation_record
    from binance.spot.bswap import bswap_request_quote
    from binance.spot.bswap import bswap_swap
    from binance.spot.bswap import bswap_swap_history



from binance.lib.utils import check_required_parameter
from binance.lib.utils import check_required_parameters


def savings_flexible_products(self, **kwargs):
    """Get Flexible Product List (USER_DATA)

    GET /sapi/v1/lending/daily/product/list

    https://binance-docs.github.io/apidocs/spot/en/#get-flexible-product-list-user_data

    Keyword Args:
        status (str, optional): "ALL", "SUBSCRIBABLE", "UNSUBSCRIBABLE"; default "ALL"
        featured (str, optional): "ALL", "true"; default "ALL"
        recvWindow (int, optional): The value cannot be greater than 60000

    """

    return self.sign_request("GET", "/sapi/v1/lending/daily/product/list", kwargs)


def savings_flexible_user_left_quota(self, productId: str, **kwargs):
    """Get Left Daily Purchase Quota of Flexible Product (USER_DATA)

    GET /sapi/v1/lending/daily/userLeftQuota

    https://binance-docs.github.io/apidocs/spot/en/#get-left-daily-purchase-quota-of-flexible-product-user_data

    Args:
        productId (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000

    """

    check_required_parameter(productId, "productId")
    payload = {"productId": productId, **kwargs}
    return self.sign_request("GET", "/sapi/v1/lending/daily/userLeftQuota", payload)


def savings_purchase_flexible_product(self, productId: str, amount: float, **kwargs):
    """POST Purchase Flexible Product (USER_DATA)

    POST /sapi/v1/lending/daily/purchase

    https://binance-docs.github.io/apidocs/spot/en/#purchase-flexible-product-user_data

    Args:
        productId (str)
        amount (float)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000

    """

    check_required_parameters([[productId, "productId"], [amount, "amount"]])
    payload = {"productId": productId, "amount": amount, **kwargs}
    return self.sign_request("POST", "/sapi/v1/lending/daily/purchase", payload)


def savings_flexible_user_redemption_quota(self, productId: str, type: str, **kwargs):
    """Get Left Daily Redemption Quota of Flexible Product (USER_DATA)

    GET /sapi/v1/lending/daily/userRedemptionQuota

    https://binance-docs.github.io/apidocs/spot/en/#get-left-daily-redemption-quota-of-flexible-product-user_data

    Args:
        productId (str)
        type (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[productId, "productId"], [type, "type"]])
    payload = {"productId": productId, "type": type, **kwargs}
    return self.sign_request(
        "GET", "/sapi/v1/lending/daily/userRedemptionQuota", payload
    )


def savings_flexible_redeem(self, productId: str, amount: float, type: str, **kwargs):
    """Redeem Flexible Product (USER_DATA)

    POST /sapi/v1/lending/daily/redeem

    https://binance-docs.github.io/apidocs/spot/en/#redeem-flexible-product-user_data

    Args:
        productId (str)
        amount (float)
        type (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [[productId, "productId"], [amount, "amount"], [type, "type"]]
    )
    payload = {"productId": productId, "amount": amount, "type": type, **kwargs}
    return self.sign_request("POST", "/sapi/v1/lending/daily/redeem", payload)


def savings_flexible_product_position(self, asset: str, **kwargs):
    """Get Flexible Product Position (USER_DATA)

    GET /sapi/v1/lending/daily/token/position

    https://binance-docs.github.io/apidocs/spot/en/#get-flexible-product-position-user_data

    Args:
        asset (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")
    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/lending/daily/token/position", payload)


def savings_project_list(self, type: str, **kwargs):
    """Get Fixed and Activity Project List(USER_DATA)

    GET /sapi/v1/lending/project/list

    https://binance-docs.github.io/apidocs/spot/en/#get-fixed-and-activity-project-list-user_data

    Args:
        type (str)
    Keyword Args:
        asset (str, optional)
        status (str, optional): "ALL", "SUBSCRIBABLE", "UNSUBSCRIBABLE"; default "ALL"
        isSortAsc (bool, optional): default "true"
        sortBy (str, optional): "START_TIME", "LOT_SIZE", "INTEREST_RATE", "DURATION"; default "START_TIME"
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10, Max:100
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(type, "type")
    payload = {"type": type, **kwargs}
    return self.sign_request("GET", "/sapi/v1/lending/project/list", payload)


def savings_purchase_project(self, projectId: str, lot: int, **kwargs):
    """Purchase Fixed/Activity Project (USER_DATA)

    POST /sapi/v1/lending/customizedFixed/purchase

    https://binance-docs.github.io/apidocs/spot/en/#purchase-fixed-activity-project-user_data

    Args:
        projectId (str)
        lot (int)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[projectId, "projectId"], [lot, "lot"]])
    payload = {"projectId": projectId, "lot": lot, **kwargs}
    return self.sign_request(
        "POST", "/sapi/v1/lending/customizedFixed/purchase", payload
    )


def savings_project_position(self, asset: str, **kwargs):
    """Get Fixed/Activity Project Position  (USER_DATA)

    GET /sapi/v1/lending/project/position/list

    https://binance-docs.github.io/apidocs/spot/en/#get-fixed-activity-project-position-user_data

    Args:
        asset (str)
    Keyword Args:
        projectId (str, optional)
        status (str, optional): "HOLDING", "REDEEMED"
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")
    payload = {"asset": asset, **kwargs}
    return self.sign_request("GET", "/sapi/v1/lending/project/position/list", payload)


def savings_account(self, **kwargs):
    """Lending Account (USER_DATA)

    GET /sapi/v1/lending/union/account

    https://binance-docs.github.io/apidocs/spot/en/#lending-account-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/lending/union/account", kwargs)


def savings_purchase_record(self, lendingType: str, **kwargs):
    """Get Purchase Record (USER_DATA)

    GET /sapi/v1/lending/union/purchaseRecord

    https://binance-docs.github.io/apidocs/spot/en/#get-purchase-record-user_data

    Args:
        lendingType (str): "DAILY" for flexible, "ACTIVITY" for activity, "CUSTOMIZED_FIXED" for fixed
    Keyword Args:
        asset (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10 Max:100
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(lendingType, "lendingType")
    payload = {"lendingType": lendingType, **kwargs}
    return self.sign_request("GET", "/sapi/v1/lending/union/purchaseRecord", payload)


def savings_redemption_record(self, lendingType: str, **kwargs):
    """Get Redemption Record (USER_DATA)

    GET /sapi/v1/lending/union/redemptionRecord

    https://binance-docs.github.io/apidocs/spot/en/#get-redemption-record-user_data

    Args:
        lendingType (str): "DAILY" for flexible, "ACTIVITY" for activity, "CUSTOMIZED_FIXED" for fixed
    Keyword Args:
        asset (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10 Max:100
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(lendingType, "lendingType")
    payload = {"lendingType": lendingType, **kwargs}
    return self.sign_request("GET", "/sapi/v1/lending/union/redemptionRecord", payload)


def savings_interest_history(self, lendingType: str, **kwargs):
    """Get Interest History (USER_DATA)

    GET /sapi/v1/lending/union/interestHistory

    https://binance-docs.github.io/apidocs/spot/en/#get-interest-history-user_data-2

    Args:
        lendingType (str): "DAILY" for flexible, "ACTIVITY" for activity, "CUSTOMIZED_FIXED" for fixed
    Keyword Args:
        asset (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Currently querying page. Start from 1. Default:1
        size (int, optional): Default:10 Max:100
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(lendingType, "lendingType")
    payload = {"lendingType": lendingType, **kwargs}
    return self.sign_request("GET", "/sapi/v1/lending/union/interestHistory", payload)


def savings_change_position(self, projectId: str, lot: int, **kwargs):
    """Change Fixed/Activity Position to Daily Position(USER_DATA)

    POST /sapi/v1/lending/positionChanged

    https://binance-docs.github.io/apidocs/spot/en/#change-fixed-activity-position-to-daily-position-user_data

    Args:
        projectId (str)
        lot (int)
    Keyword Args:
        positionId (int, optional): for fixed position
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[projectId, "projectId"], [lot, "lot"]])
    payload = {"projectId": projectId, "lot": lot, **kwargs}
    return self.sign_request("POST", "/sapi/v1/lending/positionChanged", payload)



import hmac
import json
import logging
import hashlib
from json import JSONDecodeError

import requests
from .__version__ import __version__
from binance.error import ClientError, ServerError
from binance.lib.utils import get_timestamp
from binance.lib.utils import cleanNoneValue
from binance.lib.utils import encoded_string
from binance.lib.utils import check_required_parameter


class API(object):
    """API base class

    Keyword Args:
        base_url (str, optional): the API base url, useful to switch to testnet, etc. By default it's https://api.binance.com
        timeout (int, optional): the time waiting for server response, number of seconds. https://docs.python-requests.org/en/master/user/advanced/#timeouts
        proxies (obj, optional): Dictionary mapping protocol to the URL of the proxy. e.g. {'https': 'http://1.2.3.4:8080'}
        show_limit_usage (bool, optional): whether return limit usage(requests and/or orders). By default, it's False
        show_header (bool, optional): whether return the whole response header. By default, it's False
    """

    def __init__(
        self,
        key=None,
        secret=None,
        base_url=None,
        timeout=None,
        proxies=None,
        show_limit_usage=False,
        show_header=False,
    ):
        self.key = key
        self.secret = secret
        self.timeout = timeout
        self.show_limit_usage = False
        self.show_header = False
        self.proxies = None
        self.session = requests.Session()
        self.session.headers.update(
            {
                "Content-Type": "application/json;charset=utf-8",
                "User-Agent": "binance-connector/" + __version__,
                "X-MBX-APIKEY": key,
            }
        )

        if base_url:
            self.base_url = base_url

        if show_limit_usage is True:
            self.show_limit_usage = True

        if show_header is True:
            self.show_header = True

        if type(proxies) is dict:
            self.proxies = proxies

        return

    def query(self, url_path, payload={}):
        return self.send_request("GET", url_path, payload=payload)

    def limit_request(self, http_method, url_path, payload={}):
        """limit request is for those endpoints require API key in the header"""

        check_required_parameter(self.key, "apiKey")
        return self.send_request(http_method, url_path, payload=payload)

    def sign_request(self, http_method, url_path, payload={}):
        payload["timestamp"] = get_timestamp()
        query_string = self._prepare_params(payload)
        signature = self._get_sign(query_string)
        payload["signature"] = signature
        return self.send_request(http_method, url_path, payload)

    def limited_encoded_sign_request(self, http_method, url_path, payload={}):
        """This is used for some endpoints has special symbol in the url.
        In some endpoints these symbols should not encoded
        - @
        - [
        - ]

        so we have to append those parameters in the url
        """
        payload["timestamp"] = get_timestamp()
        query_string = self._prepare_params(payload)
        signature = self._get_sign(query_string)
        url_path = url_path + "?" + query_string + "&signature=" + signature
        return self.send_request(http_method, url_path)

    def send_request(self, http_method, url_path, payload={}):
        url = self.base_url + url_path
        logging.debug("url: " + url)
        params = cleanNoneValue(
            {
                "url": url,
                "params": self._prepare_params(payload),
                "timeout": self.timeout,
                "proxies": self.proxies,
            }
        )
        response = self._dispatch_request(http_method)(**params)
        logging.debug("raw response from server:" + response.text)
        self._handle_exception(response)

        try:
            data = response.json()
        except ValueError:
            data = response.text
        result = {}

        if self.show_limit_usage:
            limit_usage = {}
            for key in response.headers.keys():
                key = key.lower()
                if (
                    key.startswith("x-mbx-used-weight")
                    or key.startswith("x-mbx-order-count")
                    or key.startswith("x-sapi-used")
                ):
                    limit_usage[key] = response.headers[key]
            result["limit_usage"] = limit_usage

        if self.show_header:
            result["header"] = response.headers

        if len(result) != 0:
            result["data"] = data
            return result

        return data

    def _prepare_params(self, params):
        return encoded_string(cleanNoneValue(params))

    def _get_sign(self, data):
        m = hmac.new(self.secret.encode("utf-8"), data.encode("utf-8"), hashlib.sha256)
        return m.hexdigest()

    def _dispatch_request(self, http_method):
        return {
            "GET": self.session.get,
            "DELETE": self.session.delete,
            "PUT": self.session.put,
            "POST": self.session.post,
        }.get(http_method, "GET")

    def _handle_exception(self, response):
        status_code = response.status_code
        if status_code < 400:
            return
        if 400 <= status_code < 500:
            try:
                err = json.loads(response.text)
            except JSONDecodeError:
                raise ClientError(status_code, None, response.text, response.headers)
            raise ClientError(status_code, err["code"], err["msg"], response.headers)
        raise ServerError(status_code, response.text)



from enum import Enum, auto


class AutoName(Enum):
    def _generate_next_value_(name, start, count, last_values):
        return name


class TransferType(AutoName):
    MAIN_C2C = auto()
    MAIN_UMFUTURE = auto()
    MAIN_CMFUTURE = auto()
    MAIN_MARGIN = auto()
    MAIN_MINING = auto()
    C2C_MAIN = auto()
    C2C_UMFUTURE = auto()
    C2C_MINING = auto()
    C2C_MARGIN = auto()
    UMFUTURE_MAIN = auto()
    UMFUTURE_C2C = auto()
    UMFUTURE_MARGIN = auto()
    CMFUTURE_MAIN = auto()
    CMFUTURE_MARGIN = auto()
    MARGIN_MAIN = auto()
    MARGIN_UMFUTURE = auto()
    MARGIN_CMFUTURE = auto()
    MARGIN_MINING = auto()
    MARGIN_C2C = auto()
    MINING_MAIN = auto()
    MINING_UMFUTURE = auto()
    MINING_C2C = auto()
    MINING_MARGIN = auto()



class Error(Exception):
    pass


class ClientError(Error):
    def __init__(self, status_code, error_code, error_message, header):
        # https status code
        self.status_code = status_code
        # error code returned from server
        self.error_code = error_code
        # error message returned from server
        self.error_message = error_message
        # the whole response header returned from server
        self.header = header


class ServerError(Error):
    def __init__(self, status_code, message):
        self.status_code = status_code
        self.message = message


class ParameterRequiredError(Error):
    def __init__(self, params):
        self.params = params

    def __str__(self):
        return "%s is mandatory, but received empty." % (", ".join(self.params))


class ParameterValueError(Error):
    def __init__(self, params):
        self.params = params

    def __str__(self):
        return "the enum value %s is invalid." % (", ".join(self.params))



from binance.lib.utils import check_required_parameter
from binance.lib.utils import check_required_parameters
from binance.lib.utils import check_enum_parameter
from binance.lib.enums import TransferType


def system_status(self):
    """System Status (System)
    Fetch system status.

    GET /sapi/v1/system/status

    https://binance-docs.github.io/apidocs/spot/en/#system-status-sapi-system
    """

    return self.query("/sapi/v1/system/status")


def coin_info(self, **kwargs):
    """All Coins' Information (USER_DATA)
    Get information of coins (available for deposit and withdraw) for user.

    GET /sapi/v1/capital/config/getall

    https://binance-docs.github.io/apidocs/spot/en/#all-coins-39-information-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/capital/config/getall", kwargs)


def account_snapshot(self, type: str, **kwargs):
    """Daily Account Snapshot (USER_DATA)

    GET /sapi/v1/accountSnapshot

    https://binance-docs.github.io/apidocs/spot/en/#daily-account-snapshot-user_data

    Parameteres:
    type -- mandatory/string -- "SPOT", "MARGIN", "FUTURES"

    Args:
        type (str): "SPOT", "MARGIN", "FUTURES"
    Keyword Args:
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): min 5, max 30, default 5
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(type, "type")
    payload = {"type": type, **kwargs}
    return self.sign_request("GET", "/sapi/v1/accountSnapshot", payload)


def disable_fast_withdraw(self, **kwargs):
    """Disable Fast Withdraw Switch (USER_DATA)

    POST /sapi/v1/account/disableFastWithdrawSwitch

    https://binance-docs.github.io/apidocs/spot/en/#disable-fast-withdraw-switch-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request(
        "POST", "/sapi/v1/account/disableFastWithdrawSwitch", kwargs
    )


def enable_fast_withdraw(self, **kwargs):
    """Enable Fast Withdraw Switch (USER_DATA)

    POST /sapi/v1/account/enableFastWithdrawSwitch

    https://binance-docs.github.io/apidocs/spot/en/#enable-fast-withdraw-switch-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request(
        "POST", "/sapi/v1/account/enableFastWithdrawSwitch", kwargs
    )


def withdraw(self, coin: str, amount: float, address: str, **kwargs):
    """Withdraw
    Submit a withdraw request.

    POST /sapi/v1/capital/withdraw/apply

    https://binance-docs.github.io/apidocs/spot/en/#withdraw-sapi

    Args:
        coin (str)
        amount (float)
        address (str)
    Keyword Args:
        withdrawOrderId (str, optional)
        network (str, optional)
        addressTag (str, optional)
        transactionFeeFlag (bool, optional)
        name (str, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [[coin, "coin"], [amount, "amount"], [address, "address"]]
    )
    payload = {"coin": coin, "amount": amount, "address": address, **kwargs}
    return self.sign_request("POST", "/sapi/v1/capital/withdraw/apply", payload)


def deposit_history(self, **kwargs):
    """Deposit History（supporting network） (USER_DATA)
    Fetch deposit history.

    GET /sapi/v1/capital/deposit/hisrec

    https://binance-docs.github.io/apidocs/spot/en/#deposit-history-supporting-network-user_data

    Keyword Args:
        coin (str, optional)
        status (int, optional): Default 0 (0:pending, 6: credited but cannot withdraw, 1:success)
        startTime (int, optional): Default: 90 days from current timestamp
        endTime (int, optional): Default: present timestamp
        offset (int, optional): Default value: 0
        limit (int, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/capital/deposit/hisrec", kwargs)


def withdraw_history(self, **kwargs):
    """Withdraw History (supporting network) (USER_DATA)
    Fetch withdraw history.

    GET /sapi/v1/capital/withdraw/history

    https://binance-docs.github.io/apidocs/spot/en/#deposit-history-user_data

    Keyword Args:
        asset (str, optional)
        status (int, optional): Default 0 (0:Email Sent,1:Cancelled, 2:Awaiting Approval,
                3:Rejected, 4:Processing, 5:Failure, 6:Completed)
        startTime (int, optional): Default: 90 days from current timestamp
        endTime (int, optional): Default: present timestamp
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/capital/withdraw/history", kwargs)


def deposit_address(self, coin: str, **kwargs):
    """Deposit Address (supporting network) (USER_DATA)
    Fetch deposit address with network.

    GET /sapi/v1/capital/deposit/address

    https://binance-docs.github.io/apidocs/spot/en/#deposit-address-supporting-network-user_data

    Keyword Args:
        coin (str, optional)
        network (str, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(coin, "coin")
    payload = {"coin": coin, **kwargs}
    return self.sign_request("GET", "/sapi/v1/capital/deposit/address", payload)


def account_status(self, **kwargs):
    """Account Status (USER_DATA)
    Fetch account status detail.

    GET /sapi/v1/account/status

    https://binance-docs.github.io/apidocs/spot/en/#account-status-sapi-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/account/status", kwargs)


def api_trading_status(self, **kwargs):
    """Account API Trading Status (USER_DATA)
    Fetch account api trading status detail.

    GET /sapi/v1/account/apiTradingStatus

    https://binance-docs.github.io/apidocs/spot/en/#account-api-trading-status-sapi-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/account/apiTradingStatus", kwargs)


def dust_log(self, **kwargs):
    """DustLog (USER_DATA)
    Fetch small amounts of assets exchanged BNB records.

    GET /sapi/v1/asset/dribblet

    https://binance-docs.github.io/apidocs/spot/en/#dustlog-sapi-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/asset/dribblet", kwargs)


def user_universal_transfer(self, type: str, asset: str, amount: str, **kwargs):
    """User Universal Transfer

    POST /sapi/v1/asset/transfer

    https://binance-docs.github.io/apidocs/spot/en/#user-universal-transfer

    Args:
        type (str)
        asset (str)
        amount (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[type, "type"], [asset, "asset"], [amount, "amount"]])
    check_enum_parameter(type, TransferType)
    payload = {"type": type, "asset": asset, "amount": amount, **kwargs}
    return self.sign_request("POST", "/sapi/v1/asset/transfer", payload)


def user_universal_transfer_history(self, type: str, **kwargs):
    """Query User Universal Transfer History

    GET /sapi/v1/asset/transfer

    https://binance-docs.github.io/apidocs/spot/en/#query-user-universal-transfer-history

     Args:
        type (str)
     Keyword Args:
        startTime (int, optional)
        endTime (int, optional)
        current (int, optional): Default 1
        size (int, optional): Default 10, Max 100
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(type, "type")
    check_enum_parameter(type, TransferType)
    payload = {"type": type, **kwargs}
    return self.sign_request("GET", "/sapi/v1/asset/transfer", payload)


def transfer_dust(self, asset, **kwargs):
    """Dust Transfer (USER_DATA)
    Convert dust assets to BNB.

    POST /sapi/v1/asset/dust

    https://binance-docs.github.io/apidocs/spot/en/#dust-transfer-user_data

    Args:
        asset (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(asset, "asset")
    payload = {"asset": asset, **kwargs}

    return self.sign_request("POST", "/sapi/v1/asset/dust", payload)


def asset_dividend_record(self, **kwargs):
    """Asset Dividend Record (USER_DATA)
    Query asset dividend record.

    GET /sapi/v1/asset/assetDividend

    https://binance-docs.github.io/apidocs/spot/en/#asset-dividend-record-user_data

    Keyword Args:
        asset (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/asset/assetDividend", kwargs)


def asset_detail(self, **kwargs):
    """Asset Detail (USER_DATA)
    Fetch details of assets supported on Binance.

    GET /sapi/v1/asset/assetDetail

    https://binance-docs.github.io/apidocs/spot/en/#asset-detail-sapi-user_data

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/asset/assetDetail", kwargs)


def trade_fee(self, **kwargs):
    """Trade Fee (USER_DATA)
    Fetch trade fee, values in percentage.

    GET /sapi/v1/asset/tradeFee

    https://binance-docs.github.io/apidocs/spot/en/#trade-fee-sapi-user_data

    Keyword Args:
        symbol (str, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/asset/tradeFee", kwargs)



import os
from setuptools import setup, find_packages


NAME = "binance-connector"
DESCRIPTION = (
    "This is a thin library that works as a connector to the Binance public API."
)
AUTHOR = "Binance"
URL = "https://github.com/binance/binance-connector-python"
VERSION = None

about = {}

with open("README.md", "r") as fh:
    about["long_description"] = fh.read()

root = os.path.abspath(os.path.dirname(__file__))

if not VERSION:
    with open(os.path.join(root, "binance", "__version__.py")) as f:
        exec(f.read(), about)
else:
    about["__version__"] = VERSION

setup(
    name=NAME,
    version=about["__version__"],
    license="MIT",
    description=DESCRIPTION,
    long_description=about["long_description"],
    long_description_content_type="text/markdown",
    AUTHOR=AUTHOR,
    url=URL,
    keywords=["Binance", "Public API"],
    install_requires=[
        "autobahn",
        "pyOpenSSL",
        "requests",
        "service-identity",
        "Twisted",
    ],
    packages=find_packages(exclude=("tests",)),
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Financial and Insurance Industry",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    python_requires=">=3.6",
)



from binance.lib.utils import check_required_parameter
from binance.lib.utils import check_required_parameters


def sub_account_create(self, subAccountString: str, **kwargs):
    """Create a Virtual Sub-account(For Master Account)
    Generate a virtual sub account under the master account

    POST /sapi/v1/sub-account/virtualSubAccount

    https://binance-docs.github.io/apidocs/spot/en/#create-a-virtual-sub-account-for-master-account

    Args:
        subAccountString (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(subAccountString, "subAccountString")
    payload = {"subAccountString": subAccountString, **kwargs}

    return self.limited_encoded_sign_request(
        "POST", "/sapi/v1/sub-account/virtualSubAccount", payload
    )


def sub_account_list(self, **kwargs):
    """Query Sub-account List(For Master Account)
    Fetch sub account list.

    GET /sapi/v1/sub-account/list

    https://binance-docs.github.io/apidocs/spot/en/#query-sub-account-list-sapi-for-master-account

    Keyword Args:
        email (str, optional): Sub-account email
        isFreeze (str, optional): true or false
        page (int, optional): default 1
        limit (int, optional): default 10, max 200
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.limited_encoded_sign_request("GET", "/sapi/v1/sub-account/list", kwargs)


def sub_account_assets(self, email: str, **kwargs):
    """Query Sub-account Assets(For Master Account)
    Fetch sub-account assets

    GET /sapi/v3/sub-account/assets

    https://binance-docs.github.io/apidocs/spot/en/#query-sub-account-assets-sapi-for-master-account

    Args:
        email (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(email, "email")
    payload = {"email": email, **kwargs}
    return self.limited_encoded_sign_request(
        "GET", "/sapi/v3/sub-account/assets", payload
    )


def sub_account_deposit_address(self, email: str, coin: str, **kwargs):
    """Get Sub-account Deposit Address (For Master Account)
    Fetch sub-account deposit address

    GET /sapi/v1/capital/deposit/subAddress

    https://binance-docs.github.io/apidocs/spot/en/#get-sub-account-deposit-address-for-master-account

    Args:
        email (str)
        coin (str)
    Keyword Args:
        network (str, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[email, "email"], [coin, "coin"]])
    payload = {"email": email, "coin": coin, **kwargs}
    return self.limited_encoded_sign_request(
        "GET", "/sapi/v1/capital/deposit/subAddress", payload
    )


def sub_account_deposit_history(self, email: str, **kwargs):
    """Get Sub-account Deposit History (For Master Account)
    Fetch sub-account deposit history

    GET /sapi/v1/capital/deposit/subHisrec

    https://binance-docs.github.io/apidocs/spot/en/#get-sub-account-deposit-address-for-master-account

    Args:
        email (str)
    Keyword Args:
        coin (str, optional)
        status (int, optional): Default 0 (0:pending,6: credited but cannot withdraw, 1:success)
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional)
        offset (int, optional): Default:0
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(email, "email")
    payload = {"email": email, **kwargs}
    return self.limited_encoded_sign_request(
        "GET", "/sapi/v1/capital/deposit/subHisrec", payload
    )


def sub_account_status(self, **kwargs):
    """Get Sub-account's Status on Margin/Futures(For Master Account)

    GET /sapi/v1/sub-account/status

    https://binance-docs.github.io/apidocs/spot/en/#get-sub-account-39-s-status-on-margin-futures-for-master-account

    Keyword Args:
        email (str, optional): Sub-account email
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.limited_encoded_sign_request(
        "GET", "/sapi/v1/sub-account/status", kwargs
    )


def sub_account_enable_margin(self, email: str, **kwargs):
    """Enable Margin for Sub-account (For Master Account)

    POST /sapi/v1/sub-account/margin/enable

    https://binance-docs.github.io/apidocs/spot/en/#enable-margin-for-sub-account-for-master-account

    Args:
        email (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(email, "email")
    payload = {"email": email, **kwargs}
    return self.limited_encoded_sign_request(
        "POST", "/sapi/v1/sub-account/margin/enable", payload
    )


def sub_account_margin_account(self, email: str, **kwargs):
    """Get Detail on Sub-account's Margin Account (For Master Account)

    GET /sapi/v1/sub-account/margin/account

    https://binance-docs.github.io/apidocs/spot/en/#get-detail-on-sub-account-39-s-margin-account-for-master-account

    Args:
        email (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(email, "email")
    payload = {"email": email, **kwargs}
    return self.limited_encoded_sign_request(
        "GET", "/sapi/v1/sub-account/margin/account", payload
    )


def sub_account_margin_account_summary(self, **kwargs):
    """Get Summary of Sub-account's Margin Account (For Master Account)

    GET /sapi/v1/sub-account/margin/accountSummary

    https://binance-docs.github.io/apidocs/spot/en/#get-summary-of-sub-account-39-s-margin-account-for-master-account

    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request(
        "GET", "/sapi/v1/sub-account/margin/accountSummary", kwargs
    )


def sub_account_enable_futures(self, email: str, **kwargs):
    """Enable Futures for Sub-account (For Master Account)

    POST /sapi/v1/sub-account/futures/enable

    https://binance-docs.github.io/apidocs/spot/en/#enable-futures-for-sub-account-for-master-account

    Args:
        email (str)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameter(email, "email")
    payload = {"email": email, **kwargs}
    return self.limited_encoded_sign_request(
        "POST", "/sapi/v1/sub-account/futures/enable", payload
    )


def sub_account_futures_transfer(
    self, email: str, asset: str, amount: float, type: int, **kwargs
):
    """Futures Transfer for Sub-account（For Master Account）

    GET /sapi/v1/sub-account/futures/transfer

    https://binance-docs.github.io/apidocs/spot/en/#get-futures-postion-risk-of-sub-account-for-master-account

    Args:
        email (str)
        asset (str)
        amount (float)
        type (int)
    """

    check_required_parameters(
        [[email, "email"], [asset, "asset"], [amount, "amount"], [type, "type"]]
    )
    payload = {"email": email, "asset": asset, "amount": amount, "type": type, **kwargs}
    return self.limited_encoded_sign_request(
        "POST", "/sapi/v1/sub-account/futures/transfer", payload
    )


def sub_account_margin_transfer(
    self, email: str, asset: str, amount: float, type: int, **kwargs
):
    """Margin Transfer for Sub-account（For Master Account)

    GET /sapi/v1/sub-account/margin/transfer

    https://binance-docs.github.io/apidocs/spot/en/#margin-transfer-for-sub-account-for-master-account

    Args:
        email (str)
        asset (str)
        amount (float)
        type (int)
    """

    check_required_parameters(
        [[email, "email"], [asset, "asset"], [amount, "amount"], [type, "type"]]
    )
    payload = {"email": email, "asset": asset, "amount": amount, "type": type, **kwargs}
    return self.limited_encoded_sign_request(
        "POST", "/sapi/v1/sub-account/margin/transfer", payload
    )


def sub_account_transfer_to_sub(
    self, toEmail: str, asset: str, amount: float, **kwargs
):
    """Transfer to Sub-account of Same Master（For Sub-account）

    POST /sapi/v1/sub-account/transfer/subToSub

    https://binance-docs.github.io/apidocs/spot/en/#transfer-to-sub-account-of-same-master-for-sub-account

    Args:
        toEmail (str)
        asset (str)
        amount (float)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters(
        [[toEmail, "toEmail"], [asset, "asset"], [amount, "amount"]]
    )
    payload = {"toEmail": toEmail, "asset": asset, "amount": amount, **kwargs}
    return self.limited_encoded_sign_request(
        "POST", "/sapi/v1/sub-account/transfer/subToSub", payload
    )


def sub_account_transfer_to_master(self, asset: str, amount: float, **kwargs):
    """Transfer to Master（For Sub-account）

    POST /sapi/v1/sub-account/transfer/subToMaster

    https://binance-docs.github.io/apidocs/spot/en/#transfer-to-master-for-sub-account

    Args:
        asset (str)
        amount (float)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[asset, "asset"], [amount, "amount"]])
    payload = {"asset": asset, "amount": amount, **kwargs}
    return self.sign_request(
        "POST", "/sapi/v1/sub-account/transfer/subToMaster", payload
    )


def sub_account_transfer_sub_account_history(self, **kwargs):
    """Sub-account Transfer History (For Sub-account)

    POST /sapi/v1/sub-account/transfer/subUserHistory

    https://binance-docs.github.io/apidocs/spot/en/#transfer-to-master-for-sub-account

    Keyword Args:
        asset (str, optional): If not sent, result of all assets will be returned
        type (int, optional): 1: transfer in, 2: transfer out
        startTime (int, optional)
        endTime (int, optional)
        limit (int, optional): Default 500
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request(
        "GET", "/sapi/v1/sub-account/transfer/subUserHistory", kwargs
    )


def sub_account_futures_asset_transfer_history(
    self, email: str, futuresType: int, **kwargs
):
    """Query Sub-account Futures Asset Transfer History(For Master Account)

    GET /sapi/v1/sub-account/futures/internalTransfer

    https://binance-docs.github.io/apidocs/spot/en/#sub-account-spot-asset-transfer-for-master-account

    Args:
        email (str): Sub-account email
        futuresType (int): 1 : USDT-maringed Futues, 2: Coin-margined Futures
    Keyword Args:
        startTime (int, optional): Default return the history with in 100 days
        endTime (int, optional): Default return the history with in 100 days
        page (int, optional): Default value: 1
        limit (int, optional): Default value: 50, Max value: 500
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters([[email, "email"], [futuresType, "futuresType"]])
    payload = {"email": email, "futuresType": futuresType, **kwargs}
    return self.sign_request(
        "GET", "/sapi/v1/sub-account/futures/internalTransfer", payload
    )


def sub_account_futures_asset_transfer(
    self,
    fromEmail: str,
    toEmail: str,
    futuresType: int,
    asset: str,
    amount: float,
    **kwargs
):
    """Query Sub-account Futures Asset Transfer History(For Master Account)

    POST /sapi/v1/sub-account/futures/internalTransfer

    https://binance-docs.github.io/apidocs/spot/en/#sub-account-futures-asset-transfer-for-master-account

    Args:
        fromEmail (str): Sender email
        toEmail (str): Recipient email
        futuresType (int): 1 : USDT-maringed Futues, 2: Coin-margined Futures
        asset (str)
        amount (float)
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters(
        [
            [fromEmail, "fromEmail"],
            [toEmail, "toEmail"],
            [futuresType, "futuresType"],
            [asset, "asset"],
            [amount, "amount"],
        ]
    )
    payload = {
        "fromEmail": fromEmail,
        "toEmail": toEmail,
        "futuresType": futuresType,
        "asset": asset,
        "amount": amount,
        **kwargs,
    }
    return self.sign_request(
        "POST", "/sapi/v1/sub-account/futures/internalTransfer", payload
    )


def sub_account_spot_summary(self, **kwargs):
    """Query Sub-account Spot Assets Summary (For Master Account)

    GET /sapi/v1/sub-account/spotSummary

    https://binance-docs.github.io/apidocs/spot/en/#query-sub-account-assets-for-master-account

    Keyword Args:
        email (str, optional): Sub account email
        page (int, optional): Default: 1
        size (int, optional): Default 10, max 20
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.sign_request("GET", "/sapi/v1/sub-account/spotSummary", kwargs)


def sub_account_universal_transfer(
    self, fromAccountType: str, toAccountType: str, asset: str, amount: float, **kwargs
):
    """Universal Transfer (For Master Account)

    POST /sapi/v1/sub-account/universalTransfer

    https://binance-docs.github.io/apidocs/spot/en/#universal-transfer-for-master-account

    You need to enable "internal transfer" option for the api key which requests this endpoint.
    Transfer from master account by default if fromEmail is not sent.
    Transfer to master account by default if toEmail is not sent.
    Transfer between futures accounts is not supported.

    Args:
        fromAccountType (str): "SPOT","USDT_FUTURE","COIN_FUTURE"
        toAccountType (str): "SPOT","USDT_FUTURE","COIN_FUTURE"
        asset (str)
        amount (float)
    Keyword Args:
        fromEmail (str, optional)
        toEmail (str, optional)
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameters(
        [
            [fromAccountType, "fromAccountType"],
            [toAccountType, "toAccountType"],
            [asset, "asset"],
            [amount, "amount"],
        ]
    )

    payload = {
        "fromAccountType": fromAccountType,
        "toAccountType": toAccountType,
        "asset": asset,
        "amount": amount,
        **kwargs,
    }
    return self.limited_encoded_sign_request(
        "POST", "/sapi/v1/sub-account/universalTransfer", payload
    )


def sub_account_universal_transfer_history(self, **kwargs):
    """Query Universal Transfer History (For Master Account)

    GET /sapi/v1/sub-account/universalTransfer

    https://binance-docs.github.io/apidocs/spot/en/#query-universal-transfer-history-for-master-account

    fromEmail and toEmail cannot be sent at the same time.
    Return fromEmail equal master account email by default.
    Only get the latest history of past 30 days.

    Keyword Args:
        fromEmail (str, optional)
        toEmail (str, optional)
        startTime (int, optional)
        endTime (int, optional)
        page (int, optional)
        limit (int, optional): Default 10, max 20
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.limited_encoded_sign_request(
        "GET", "/sapi/v1/sub-account/universalTransfer", kwargs
    )


def sub_account_futures_account(self, email: str, futuresType: int, **kwargs):
    """Get Detail on Sub-account's Futures Account V2 (For Master Account)

    GET /sapi/v2/sub-account/futures/account

    https://binance-docs.github.io/apidocs/spot/en/#get-detail-on-sub-account-39-s-futures-account-v2-for-master-account

    Args:
        email (str): Sub-account email
        futuresType (int): 1 : USDT-maringed Futues, 2: Coin-margined Futures
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[email, "email"], [futuresType, "futuresType"]])

    payload = {"email": email, "futuresType": futuresType, **kwargs}
    return self.limited_encoded_sign_request(
        "GET", "/sapi/v2/sub-account/futures/account", payload
    )


def sub_account_futures_account_summary(self, futuresType: int, **kwargs):
    """Get Summary of Sub-account's Futures Account V2 (For Master Account)

    GET /sapi/v2/sub-account/futures/accountSummary

    https://binance-docs.github.io/apidocs/spot/en/#get-detail-on-sub-account-39-s-futures-account-v2-for-master-account

    Args:
        futuresType (int): 1 : USDT-maringed Futues, 2: Coin-margined Futures
    Keyword Args:
        page (int, optional): Default:1
        limit (int, optional): Default 10, max 20
        recvWindow (int, optional): The value cannot be greater than 60000
    """
    check_required_parameter(futuresType, "futuresType")

    payload = {"futuresType": futuresType, **kwargs}

    return self.sign_request(
        "GET", "/sapi/v2/sub-account/futures/accountSummary", payload
    )


def sub_account_futures_position_risk(self, email: str, futuresType: str, **kwargs):
    """Get Futures Position-Risk of Sub-account V2 (For Master Account)

    GET /sapi/v2/sub-account/futures/positionRisk

    https://binance-docs.github.io/apidocs/spot/en/#get-futures-position-risk-of-sub-account-v2-for-master-account

    Args:
        email (str): Sub-account email
        futuresType (int): 1 : USDT-maringed Futues, 2: Coin-margined Futures
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[email, "email"], [futuresType, "futuresType"]])

    payload = {"email": email, "futuresType": futuresType, **kwargs}
    return self.limited_encoded_sign_request(
        "GET", "/sapi/v2/sub-account/futures/positionRisk", payload
    )


def sub_account_spot_transfer_history(self, **kwargs):
    """Query Sub-account Spot Asset Transfer History (SAPI For Master Account)

    GET /sapi/v1/sub-account/sub/transfer/history

    https://binance-docs.github.io/apidocs/spot/en/#query-sub-account-spot-asset-transfer-history-for-master-account

    Keyword Args:
        fromEmail (str, optional)
        toEmail (str, optional)
        startTime (int, optional): Default return the history with in 100 days
        endTime (int, optional): Default return the history with in 100 days
        page (int, optional): Default value: 1
        limit (int, optional): Default value: 500
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    return self.limited_encoded_sign_request(
        "GET", "/sapi/v1/sub-account/sub/transfer/history", kwargs
    )


def sub_account_enable_leverage_token(self, email: str, enableBlvt: bool, **kwargs):
    """Enable Leverage Token for Sub-account(For Master Account)
    Enable leverage token for sub-account

    POST /sapi/v1/sub-account/blvt/enable

    https://binance-docs.github.io/apidocs/spot/en/#enable-leverage-token-for-sub-account-for-master-account

    Args:
        email (str): Sub-account email
        enableBlvt (bool): Only true for now
    Keyword Args:
        recvWindow (int, optional): The value cannot be greater than 60000
    """

    check_required_parameters([[email, "email"], [enableBlvt, "enableBlvt"]])
    payload = {"email": email, "enableBlvt": enableBlvt, **kwargs}

    return self.limited_encoded_sign_request(
        "POST", "/sapi/v1/sub-account/blvt/enable", payload
    )



import json
from twisted.internet import reactor
from binance.websocket.binance_socket_manager import BinanceSocketManager


class BinanceWebsocketClient(BinanceSocketManager):
    def __init__(self, stream_url):
        super().__init__(stream_url)

    def stop(self):
        try:
            self.close()
        finally:
            reactor.stop()

    def _single_stream(self, stream):
        if isinstance(stream, str):
            return True
        elif isinstance(stream, list):
            return False
        else:
            raise ValueError("Invalid stream name, expect string or array")

    def live_subscribe(self, stream, id, callback, **kwargs):
        """live subscribe websocket
        Connect to the server
        - SPOT: wss://stream.binance.com:9443/ws
        - SPOT testnet : wss://testnet.binance.vision/ws

        and sending the subscribe message, e.g.

        {"method": "SUBSCRIBE","params":["btcusdt@miniTicker"],"id": 100}

        """
        combined = False
        if self._single_stream(stream):
            stream = [stream]
        else:
            combined = True

        data = {"method": "SUBSCRIBE", "params": stream, "id": id}

        data.update(**kwargs)
        payload = json.dumps(data, ensure_ascii=False).encode("utf8")
        stream_name = "-".join(stream)
        return self._start_socket(
            stream_name, payload, callback, is_combined=combined, is_live=True
        )

    def instant_subscribe(self, stream, callback, **kwargs):
        """Instant subscribe, e.g.
        wss://stream.binance.com:9443/ws/btcusdt@bookTicker
        wss://stream.binance.com:9443/stream?streams=btcusdt@bookTicker/bnbusdt@bookTicker

        """
        combined = False
        if not self._single_stream(stream):
            combined = True
            stream = "/".join(stream)

        data = {"method": "SUBSCRIBE", "params": stream}

        data.update(**kwargs)
        payload = json.dumps(data, ensure_ascii=False).encode("utf8")
        stream_name = "-".join(stream)
        return self._start_socket(
            stream_name, payload, callback, is_combined=combined, is_live=False
        )
